# Copyright 2017 AT&T Corporation.
# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from tempest.lib.services.network import networks_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestNetworksClient(base.BaseServiceTest):

    FAKE_NETWORKS = {
        "networks": [
            {
                "admin_state_up": True,
                "availability_zone_hints": [],
                "availability_zones": [
                    "nova"
                ],
                "created_at": "2016-03-08T20:19:41",
                "dns_domain": "my-domain.org.",
                "id": "d32019d3-bc6e-4319-9c1d-6722fc136a22",
                "ipv4_address_scope": None,
                "ipv6_address_scope": None,
                "l2_adjacency": False,
                "mtu": 0,
                "name": "net1",
                "port_security_enabled": True,
                "project_id": "4fd44f30292945e481c7b8a0c8908869",
                "qos_policy_id": "6a8454ade84346f59e8d40665f878b2e",
                "revision_number": 1,
                "router:external": False,
                "shared": False,
                "status": "ACTIVE",
                "subnets": [
                        "54d6f61d-db07-451c-9ab3-b9609b6b6f0b"
                ],
                "tenant_id": "4fd44f30292945e481c7b8a0c8908869",
                "updated_at": "2016-03-08T20:19:41",
                "vlan_transparent": True,
                "description": "",
                "is_default": False
            },
            {
                "admin_state_up": True,
                "availability_zone_hints": [],
                "availability_zones": [
                    "nova"
                ],
                "created_at": "2016-03-08T20:19:41",
                "dns_domain": "my-domain.org.",
                "id": "db193ab3-96e3-4cb3-8fc5-05f4296d0324",
                "ipv4_address_scope": None,
                "ipv6_address_scope": None,
                "l2_adjacency": False,
                "mtu": 0,
                "name": "net2",
                "port_security_enabled": True,
                "project_id": "26a7980765d0414dbc1fc1f88cdb7e6e",
                "qos_policy_id": "bfdb6c39f71e4d44b1dfbda245c50819",
                "revision_number": 3,
                "router:external": False,
                "shared": False,
                "status": "ACTIVE",
                "subnets": [
                        "08eae331-0402-425a-923c-34f7cfe39c1b"
                ],
                "tenant_id": "26a7980765d0414dbc1fc1f88cdb7e6e",
                "updated_at": "2016-03-08T20:19:41",
                "vlan_transparent": False,
                "description": "",
                "is_default": False
            }
        ]
    }

    FAKE_NETWORK_ID = "d32019d3-bc6e-4319-9c1d-6722fc136a22"

    FAKE_NETWORK1 = {
        "name": "net1",
        "admin_state_up": True,
        "qos_policy_id": "6a8454ade84346f59e8d40665f878b2e"
    }

    FAKE_NETWORK2 = {
        "name": "net2",
        "admin_state_up": True,
        "qos_policy_id": "bfdb6c39f71e4d44b1dfbda245c50819"
    }

    FAKE_NETWORKS_REQ = {
        "networks": [
            FAKE_NETWORK1,
            FAKE_NETWORK2
        ]
    }

    FAKE_DHCP_AGENT_NETWORK_ID = "80515c45-651f-4f9a-b82b-2ca8a7301a8d"

    FAKE_DHCP_AGENTS = {
        "agents": [
            {
                "binary": "neutron-dhcp-agent",
                "description": None,
                "admin_state_up": True,
                "heartbeat_timestamp": "2017-06-22 18:29:50",
                "availability_zone": "nova",
                "alive": True,
                "topic": "dhcp_agent",
                "host": "osboxes",
                "ha_state": None,
                "agent_type": "DHCP agent",
                "resource_versions": {},
                "created_at": "2017-06-19 21:39:51",
                "started_at": "2017-06-19 21:39:51",
                "id": "b6cfb7a1-6ac4-4980-993c-9d295d37062e",
                "configurations": {
                    "subnets": 2,
                    "dhcp_lease_duration": 86400,
                    "dhcp_driver": "neutron.agent.linux.dhcp.Dnsmasq",
                    "networks": 1,
                    "log_agent_heartbeats": False,
                    "ports": 3
                }
            }
        ]
    }

    def setUp(self):
        super(TestNetworksClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.networks_client = networks_client.NetworksClient(
            fake_auth, "network", "regionOne")

    def _test_list_networks(self, bytes_body=False):
        self.check_service_client_function(
            self.networks_client.list_networks,
            "tempest.lib.common.rest_client.RestClient.get",
            self.FAKE_NETWORKS,
            bytes_body,
            200)

    def _test_create_network(self, bytes_body=False):
        self.check_service_client_function(
            self.networks_client.create_network,
            "tempest.lib.common.rest_client.RestClient.post",
            {"network": self.FAKE_NETWORKS["networks"][0]},
            bytes_body,
            201,
            **self.FAKE_NETWORK1)

    def _test_create_bulk_networks(self, bytes_body=False):
        self.check_service_client_function(
            self.networks_client.create_bulk_networks,
            "tempest.lib.common.rest_client.RestClient.post",
            self.FAKE_NETWORKS,
            bytes_body,
            201,
            networks=self.FAKE_NETWORKS_REQ)

    def _test_show_network(self, bytes_body=False):
        self.check_service_client_function(
            self.networks_client.show_network,
            "tempest.lib.common.rest_client.RestClient.get",
            {"network": self.FAKE_NETWORKS["networks"][0]},
            bytes_body,
            200,
            network_id=self.FAKE_NETWORK_ID)

    def _test_update_network(self, bytes_body=False):
        update_kwargs = {
            "name": "sample_network_5_updated",
            "qos_policy_id": "6a8454ade84346f59e8d40665f878b2e"
        }

        resp_body = {
            "network": copy.deepcopy(
                self.FAKE_NETWORKS["networks"][0]
            )
        }
        resp_body["network"].update(update_kwargs)

        self.check_service_client_function(
            self.networks_client.update_network,
            "tempest.lib.common.rest_client.RestClient.put",
            resp_body,
            bytes_body,
            200,
            network_id=self.FAKE_NETWORK_ID,
            **update_kwargs)

    def _test_list_dhcp_agents_on_hosting_network(self, bytes_body=False):
        self.check_service_client_function(
            self.networks_client.list_dhcp_agents_on_hosting_network,
            "tempest.lib.common.rest_client.RestClient.get",
            self.FAKE_DHCP_AGENTS,
            bytes_body,
            200,
            network_id=self.FAKE_DHCP_AGENT_NETWORK_ID)

    def test_delete_network(self):
        self.check_service_client_function(
            self.networks_client.delete_network,
            "tempest.lib.common.rest_client.RestClient.delete",
            {},
            status=204,
            network_id=self.FAKE_NETWORK_ID)

    def test_list_networks_with_str_body(self):
        self._test_list_networks()

    def test_list_networks_with_bytes_body(self):
        self._test_list_networks(bytes_body=True)

    def test_create_network_with_str_body(self):
        self._test_create_network()

    def test_create_network_with_bytes_body(self):
        self._test_create_network(bytes_body=True)

    def test_create_bulk_network_with_str_body(self):
        self._test_create_bulk_networks()

    def test_create_bulk_network_with_bytes_body(self):
        self._test_create_bulk_networks(bytes_body=True)

    def test_show_network_with_str_body(self):
        self._test_show_network()

    def test_show_network_with_bytes_body(self):
        self._test_show_network(bytes_body=True)

    def test_update_network_with_str_body(self):
        self._test_update_network()

    def test_update_network_with_bytes_body(self):
        self._test_update_network(bytes_body=True)

    def test_list_dhcp_agents_on_hosting_network_with_str_body(self):
        self._test_list_dhcp_agents_on_hosting_network()

    def test_list_dhcp_agents_on_hosting_network_with_bytes_body(self):
        self._test_list_dhcp_agents_on_hosting_network(bytes_body=True)



# Copyright 2015 Fujitsu(fnst) Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute import base
from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators


CONF = config.CONF


class VolumesSnapshotsTestJSON(base.BaseV2ComputeTest):
    """Test volume snapshots with compute microversion less than 2.36"""

    # These tests will fail with a 404 starting from microversion 2.36. For
    # more information, see:
    # https://docs.openstack.org/api-ref/compute/#volume-extension-os-volumes-os-snapshots-deprecated
    max_microversion = '2.35'

    @classmethod
    def skip_checks(cls):
        super(VolumesSnapshotsTestJSON, cls).skip_checks()
        if not CONF.service_available.cinder:
            skip_msg = ("%s skipped as Cinder is not available" % cls.__name__)
            raise cls.skipException(skip_msg)
        if not CONF.volume_feature_enabled.snapshot:
            skip_msg = ("Cinder volume snapshots are disabled")
            raise cls.skipException(skip_msg)

    @classmethod
    def setup_clients(cls):
        super(VolumesSnapshotsTestJSON, cls).setup_clients()
        cls.volumes_client = cls.volumes_extensions_client
        cls.snapshots_client = cls.snapshots_extensions_client

    @decorators.idempotent_id('cd4ec87d-7825-450d-8040-6e2068f2da8f')
    def test_volume_snapshot_create_get_list_delete(self):
        """Test create/get/list/delete volume snapshot"""
        volume = self.create_volume()
        self.addCleanup(self.delete_volume, volume['id'])

        s_name = data_utils.rand_name(self.__class__.__name__ + '-Snapshot')
        # Create snapshot
        snapshot = self.snapshots_client.create_snapshot(
            volume_id=volume['id'],
            display_name=s_name)['snapshot']

        def delete_snapshot(snapshot_id):
            waiters.wait_for_volume_resource_status(self.snapshots_client,
                                                    snapshot_id,
                                                    'available')
            # Delete snapshot
            self.snapshots_client.delete_snapshot(snapshot_id)
            self.snapshots_client.wait_for_resource_deletion(snapshot_id)

        self.addCleanup(delete_snapshot, snapshot['id'])
        self.assertEqual(volume['id'], snapshot['volumeId'])
        # Get snapshot
        fetched_snapshot = self.snapshots_client.show_snapshot(
            snapshot['id'])['snapshot']
        self.assertEqual(s_name, fetched_snapshot['displayName'])
        self.assertEqual(volume['id'], fetched_snapshot['volumeId'])
        # Fetch all snapshots
        snapshots = self.snapshots_client.list_snapshots()['snapshots']
        self.assertIn(snapshot['id'], map(lambda x: x['id'], snapshots))



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from tempest.lib.services.compute import baremetal_nodes_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestBareMetalNodesClient(base.BaseServiceTest):

    FAKE_NODE_INFO = {'cpus': '8',
                      'disk_gb': '64',
                      'host': '10.0.2.15',
                      'id': 'Identifier',
                      'instance_uuid': "null",
                      'interfaces': [
                          {
                              "address": "20::01",
                              "datapath_id": "null",
                              "id": 1,
                              "port_no": None
                          }
                      ],
                      'memory_mb': '8192',
                      'task_state': None}

    def setUp(self):
        super(TestBareMetalNodesClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.baremetal_nodes_client = (baremetal_nodes_client.
                                       BaremetalNodesClient
                                       (fake_auth, 'compute',
                                        'regionOne'))

    def _test_bareMetal_nodes(self, operation='list', bytes_body=False):
        if operation != 'list':
            expected = {"node": self.FAKE_NODE_INFO}
            function = self.baremetal_nodes_client.show_baremetal_node
        else:
            node_info = copy.deepcopy(self.FAKE_NODE_INFO)
            del node_info['instance_uuid']
            expected = {"nodes": [node_info]}
            function = self.baremetal_nodes_client.list_baremetal_nodes

        self.check_service_client_function(
            function,
            'tempest.lib.common.rest_client.RestClient.get',
            expected, bytes_body, 200,
            baremetal_node_id='Identifier')

    def test_list_bareMetal_nodes_with_str_body(self):
        self._test_bareMetal_nodes()

    def test_list_bareMetal_nodes_with_bytes_body(self):
        self._test_bareMetal_nodes(bytes_body=True)

    def test_show_bareMetal_node_with_str_body(self):
        self._test_bareMetal_nodes('show')

    def test_show_bareMetal_node_with_bytes_body(self):
        self._test_bareMetal_nodes('show', True)



# Copyright 2018 FiberHome Telecommunication Technologies CO.,LTD
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.volume import base
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc


class VolumeServicesNegativeTest(base.BaseVolumeAdminTest):
    """Negative tests of volume services"""

    @classmethod
    def resource_setup(cls):
        super(VolumeServicesNegativeTest, cls).resource_setup()
        services = cls.admin_volume_services_client.list_services()['services']
        cls.host = services[0]['host']
        cls.binary = services[0]['binary']

    @decorators.attr(type='negative')
    @decorators.idempotent_id('3246ce65-ba70-4159-aa3b-082c28e4b484')
    def test_enable_service_with_invalid_host(self):
        """Test enabling volume service with invalid host should fail"""
        self.assertRaises(lib_exc.NotFound,
                          self.admin_volume_services_client.enable_service,
                          host='invalid_host', binary=self.binary)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('c571f179-c6e6-4c50-a0ab-368b628a8ac1')
    def test_disable_service_with_invalid_binary(self):
        """Test disabling volume service with invalid binary should fail"""
        self.assertRaises(lib_exc.NotFound,
                          self.admin_volume_services_client.disable_service,
                          host=self.host, binary='invalid_binary')

    @decorators.attr(type='negative')
    @decorators.idempotent_id('77767b36-5e8f-4c68-a0b5-2308cc21ec64')
    def test_disable_log_reason_with_no_reason(self):
        """Test disabling volume service with none reason should fail"""
        self.assertRaises(lib_exc.BadRequest,
                          self.admin_volume_services_client.disable_log_reason,
                          host=self.host, binary=self.binary,
                          disabled_reason=None)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('712bfab8-1f44-4eb5-a632-fa70bf78f05e')
    def test_freeze_host_with_invalid_host(self):
        """Test freezing volume service with invalid host should fail"""
        self.assertRaises(lib_exc.BadRequest,
                          self.admin_volume_services_client.freeze_host,
                          host='invalid_host')

    @decorators.attr(type='negative')
    @decorators.idempotent_id('7c6287c9-d655-47e1-9a11-76f6657a6dce')
    def test_thaw_host_with_invalid_host(self):
        """Test thawing volume service with invalid host should fail"""
        self.assertRaises(lib_exc.BadRequest,
                          self.admin_volume_services_client.thaw_host,
                          host='invalid_host')



# Copyright 2016 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import time
from unittest import mock


from tempest.lib.common import thread
from tempest.lib.common.utils import test_utils
from tempest.lib import exceptions
from tempest.tests import base


class TestTestUtils(base.TestCase):

    def test_find_test_caller_test_case(self):
        # Calling it from here should give us the method we're in.
        self.assertEqual('TestTestUtils:test_find_test_caller_test_case',
                         test_utils.find_test_caller())

    def test_find_test_caller_setup_self(self):
        def setUp(self):
            return test_utils.find_test_caller()
        self.assertEqual('TestTestUtils:setUp', setUp(self))

    def test_find_test_caller_setup_no_self(self):
        def setUp():
            return test_utils.find_test_caller()
        self.assertEqual(':setUp', setUp())

    def test_find_test_caller_setupclass_cls(self):
        def setUpClass(cls):  # noqa
            return test_utils.find_test_caller()
        self.assertEqual('TestTestUtils:setUpClass',
                         setUpClass(self.__class__))

    def test_find_test_caller_teardown_self(self):
        def tearDown(self):
            return test_utils.find_test_caller()
        self.assertEqual('TestTestUtils:tearDown', tearDown(self))

    def test_find_test_caller_teardown_no_self(self):
        def tearDown():
            return test_utils.find_test_caller()
        self.assertEqual(':tearDown', tearDown())

    def test_find_test_caller_teardown_class(self):
        def tearDownClass(cls):  # noqa
            return test_utils.find_test_caller()
        self.assertEqual('TestTestUtils:tearDownClass',
                         tearDownClass(self.__class__))

    def test_call_and_ignore_notfound_exc_when_notfound_raised(self):
        def raise_not_found():
            raise exceptions.NotFound()
        self.assertIsNone(
            test_utils.call_and_ignore_notfound_exc(raise_not_found))

    def test_call_and_ignore_notfound_exc_when_value_error_raised(self):
        def raise_value_error():
            raise ValueError()
        self.assertRaises(ValueError, test_utils.call_and_ignore_notfound_exc,
                          raise_value_error)

    def test_call_and_ignore_notfound_exc_when_serverfault_raised(self):
        calls = []

        def raise_serverfault():
            calls.append('call')
            raise exceptions.ServerFault()
        self.assertRaises(exceptions.ServerFault,
                          test_utils.call_and_ignore_notfound_exc,
                          raise_serverfault)
        self.assertEqual(3, len(calls))

    def test_call_and_ignore_notfound_exc(self):
        m = mock.Mock(return_value=42)
        args, kwargs = (1,), {'1': None}
        self.assertEqual(
            42, test_utils.call_and_ignore_notfound_exc(m, *args, **kwargs))
        m.assert_called_once_with(*args, **kwargs)


class TestCallUntilTrue(base.TestCase):

    def test_call_until_true_when_true_at_first_call(self):
        """func returns True at first call

        """
        self._test_call_until_true(return_values=[True],
                                   duration=30.,
                                   time_sequence=[10., 60.])

    def test_call_until_true_when_true_before_timeout(self):
        """func returns false at first call, then True before timeout

        """
        self._test_call_until_true(return_values=[False, True],
                                   duration=30.,
                                   time_sequence=[10., 39., 41.])

    def test_call_until_true_when_never_true_before_timeout(self):
        """func returns false, then false, just before timeout

        """
        self._test_call_until_true(return_values=[False, False],
                                   duration=30.,
                                   time_sequence=[10., 39., 41.])

    def test_call_until_true_with_params(self):
        """func is called using given parameters

        """
        self._test_call_until_true(return_values=[False, True],
                                   duration=30.,
                                   time_sequence=[10., 30., 60.],
                                   args=(1, 2),
                                   kwargs=dict(foo='bar', bar='foo'))

    def _test_call_until_true(self, return_values, duration, time_sequence,
                              args=None, kwargs=None):
        """Test call_until_true function

        :param return_values: list of booleans values to be returned
        each time given function is called. If any of these values
        is not consumed by calling the function the test fails.
        The list must contain a sequence of False items terminated
        by a single True or False
        :param duration: parameter passed to call_until_true function
        (a floating point value).
        :param time_sequence: sequence of time values returned by
        mocked time.time function used to trigger call_until_true
        behavior when handling timeout condition. The sequence must
        contain the exact number of values expected to be consumed by
        each time call_until_true calls time.time function.
        :param args: sequence of positional arguments to be passed
        to call_until_true function.
        :param kwargs: sequence of named arguments to be passed
        to call_until_true function.
        """

        # all values except the last are False
        self.assertEqual([False] * len(return_values[:-1]), return_values[:-1])
        # last value can be True or False
        self.assertIn(return_values[-1], [True, False])

        # GIVEN
        func = mock.Mock(side_effect=return_values)
        sleep = 10.  # this value has no effect as time.sleep is being mocked
        sleep_func = self.patch('time.sleep')
        time_func = self._patch_time(time_sequence)
        args = args or tuple()
        kwargs = kwargs or dict()

        # WHEN
        result = test_utils.call_until_true(func, duration, sleep,
                                            *args, **kwargs)
        # THEN

        # It must return last returned value
        self.assertIs(return_values[-1], result)

        self._test_func_calls(func, return_values, *args, **kwargs)
        self._test_sleep_calls(sleep_func, return_values, sleep)
        # The number of times time.time is called is not relevant as a
        # requirement of call_until_true. What is instead relevant is that
        # call_until_true use a mocked function to make the test reliable
        # and the test actually provide the right sequence of numbers to
        # reproduce the behavior has to be tested
        self._assert_called_n_times(time_func, len(time_sequence))

    def _patch_time(self, time_sequence):
        # Iterator over time sequence
        time_iterator = iter(time_sequence)
        # Preserve original time.time() behavior for other threads
        original_time = time.time
        thread_id = thread.get_ident()

        def mocked_time():
            if thread.get_ident() == thread_id:
                # Test thread => return time sequence values
                return next(time_iterator)
            else:
                # Other threads => call original time function
                return original_time()

        return self.patch('time.time', side_effect=mocked_time)

    def _test_func_calls(self, func, return_values, *args, **kwargs):
        self._assert_called_n_times(func, len(return_values), *args, **kwargs)

    def _test_sleep_calls(self, sleep_func, return_values, sleep):
        # count first consecutive False
        expected_count = 0
        for value in return_values:
            if value:
                break
            expected_count += 1
        self._assert_called_n_times(sleep_func, expected_count, sleep)

    def _assert_called_n_times(self, mock_func, expected_count, *args,
                               **kwargs):
        calls = [mock.call(*args, **kwargs)] * expected_count
        self.assertEqual(expected_count, mock_func.call_count)
        mock_func.assert_has_calls(calls)



# Copyright 2017 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.object_storage import bulk_middleware_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestBulkMiddlewareClient(base.BaseServiceTest):

    def setUp(self):
        super(TestBulkMiddlewareClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = bulk_middleware_client.BulkMiddlewareClient(
            fake_auth, 'object-storage', 'regionOne')

    def test_upload_archive(self):
        url = 'test_path?extract-archive=tar'
        data = 'test_data'
        self.check_service_client_function(
            self.client.upload_archive,
            'tempest.lib.common.rest_client.RestClient.put',
            {},
            mock_args=[url, data, {}],
            resp_as_string=True,
            upload_path='test_path', data=data, archive_file_format='tar')

    def test_delete_bulk_data(self):
        url = '?bulk-delete'
        data = 'test_data'
        self.check_service_client_function(
            self.client.delete_bulk_data,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            mock_args=[url, {}, data],
            resp_as_string=True,
            data=data)

    def _test_delete_bulk_data_with_post(self, status):
        url = '?bulk-delete'
        data = 'test_data'
        self.check_service_client_function(
            self.client.delete_bulk_data_with_post,
            'tempest.lib.common.rest_client.RestClient.post',
            {},
            mock_args=[url, data, {}],
            resp_as_string=True,
            status=status,
            data=data)

    def test_delete_bulk_data_with_post_200(self):
        self._test_delete_bulk_data_with_post(200)

    def test_delete_bulk_data_with_post_204(self):
        self._test_delete_bulk_data_with_post(204)



# Copyright (C) 2013 eNovance SAS <licensing@enovance.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.api.object_storage import base
from tempest.common import custom_matchers
from tempest.common import utils
from tempest.lib import decorators


class CrossdomainTest(base.BaseObjectTest):
    """Test crossdomain policy"""

    @classmethod
    def resource_setup(cls):
        super(CrossdomainTest, cls).resource_setup()

        cls.xml_start = '<?xml version="1.0"?>\n' \
                        '<!DOCTYPE cross-domain-policy SYSTEM ' \
                        '"http://www.adobe.com/xml/dtds/cross-domain-policy.' \
                        'dtd" >\n<cross-domain-policy>\n'

        cls.xml_end = "</cross-domain-policy>"

    @decorators.idempotent_id('d1b8b031-b622-4010-82f9-ff78a9e915c7')
    @utils.requires_ext(extension='crossdomain', service='object')
    def test_get_crossdomain_policy(self):
        """Test getting crossdomain policy"""
        url = self.account_client._get_base_version_url() + "crossdomain.xml"
        resp, body = self.account_client.raw_request(url, "GET")
        self.account_client._error_checker(resp, body)
        body = body.decode()

        self.assertTrue(body.startswith(self.xml_start) and
                        body.endswith(self.xml_end))

        # The target of the request is not any Swift resource. Therefore, the
        # existence of response header is checked without a custom matcher.
        self.assertIn('content-length', resp)
        self.assertIn('content-type', resp)
        self.assertIn('x-trans-id', resp)
        self.assertIn('date', resp)
        # Check only the format of common headers with custom matcher
        self.assertThat(resp, custom_matchers.AreAllWellFormatted())



# Copyright (C) 2014 eNovance SAS <licensing@enovance.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.api.network import base
from tempest.common import utils
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators


class MeteringTestJSON(base.BaseAdminNetworkTest):
    """Tests the following operations in the Neutron API:

        List, Show, Create, Delete Metering labels
        List, Show, Create, Delete Metering labels rules
    """

    @classmethod
    def skip_checks(cls):
        super(MeteringTestJSON, cls).skip_checks()
        if not utils.is_extension_enabled('metering', 'network'):
            msg = "metering extension not enabled."
            raise cls.skipException(msg)

    @classmethod
    def resource_setup(cls):
        super(MeteringTestJSON, cls).resource_setup()
        description = "metering label created by tempest"
        name = data_utils.rand_name("metering-label")
        cls.metering_label = cls.create_metering_label(name, description)
        remote_ip_prefix = ("10.0.0.0/24" if cls._ip_version == 4
                            else "fd02::/64")
        direction = "ingress"
        cls.metering_label_rule = cls.create_metering_label_rule(
            remote_ip_prefix, direction,
            metering_label_id=cls.metering_label['id'])

    @classmethod
    def create_metering_label(cls, name, description):
        """Wrapper utility that returns a test metering label."""
        body = cls.admin_metering_labels_client.create_metering_label(
            description=description,
            name=name)
        metering_label = body['metering_label']
        cls.addClassResourceCleanup(
            test_utils.call_and_ignore_notfound_exc,
            cls.admin_metering_labels_client.delete_metering_label,
            metering_label['id'])
        return metering_label

    @classmethod
    def create_metering_label_rule(cls, remote_ip_prefix, direction,
                                   metering_label_id):
        """Wrapper utility that returns a test metering label rule."""
        client = cls.admin_metering_label_rules_client
        body = client.create_metering_label_rule(
            remote_ip_prefix=remote_ip_prefix, direction=direction,
            metering_label_id=metering_label_id)
        metering_label_rule = body['metering_label_rule']
        cls.addClassResourceCleanup(
            test_utils.call_and_ignore_notfound_exc,
            client.delete_metering_label_rule, metering_label_rule['id'])
        return metering_label_rule

    def _delete_metering_label(self, metering_label_id):
        # Deletes a label and verifies if it is deleted or not
        self.admin_metering_labels_client.delete_metering_label(
            metering_label_id)
        # Asserting that the label is not found in list after deletion
        labels = self.admin_metering_labels_client.list_metering_labels(
            id=metering_label_id)
        self.assertEmpty(labels['metering_labels'])

    def _delete_metering_label_rule(self, metering_label_rule_id):
        client = self.admin_metering_label_rules_client
        # Deletes a rule and verifies if it is deleted or not
        client.delete_metering_label_rule(metering_label_rule_id)
        # Asserting that the rule is not found in list after deletion
        rules = client.list_metering_label_rules(id=metering_label_rule_id)
        self.assertEqual(len(rules['metering_label_rules']), 0)

    @decorators.idempotent_id('e2fb2f8c-45bf-429a-9f17-171c70444612')
    def test_list_metering_labels(self):
        """Verify listing metering labels"""
        body = self.admin_metering_labels_client.list_metering_labels(id=33)
        metering_labels = body['metering_labels']
        self.assertEmpty(metering_labels)

    @decorators.idempotent_id('ec8e15ff-95d0-433b-b8a6-b466bddb1e50')
    def test_create_delete_metering_label_with_filters(self):
        """Verifies creating and deleting metering label with filters"""
        # Creates a label
        name = data_utils.rand_name('metering-label-')
        description = "label created by tempest"
        body = self.admin_metering_labels_client.create_metering_label(
            name=name, description=description)
        metering_label = body['metering_label']
        self.addCleanup(self._delete_metering_label,
                        metering_label['id'])
        # Assert whether created labels are found in labels list or fail
        # if created labels are not found in labels list
        labels = (self.admin_metering_labels_client.list_metering_labels(
                  id=metering_label['id']))
        self.assertEqual(len(labels['metering_labels']), 1)

    @decorators.idempotent_id('30abb445-0eea-472e-bd02-8649f54a5968')
    def test_show_metering_label(self):
        """Verifies the details of a metering label"""
        body = self.admin_metering_labels_client.show_metering_label(
            self.metering_label['id'])
        metering_label = body['metering_label']
        self.assertEqual(self.metering_label['id'], metering_label['id'])
        self.assertEqual(self.metering_label['tenant_id'],
                         metering_label['project_id'])
        self.assertEqual(self.metering_label['name'], metering_label['name'])
        self.assertEqual(self.metering_label['description'],
                         metering_label['description'])

    @decorators.idempotent_id('cc832399-6681-493b-9d79-0202831a1281')
    def test_list_metering_label_rules(self):
        """Verifies listing metering label rules"""
        client = self.admin_metering_label_rules_client
        # Verify rule filtering
        body = client.list_metering_label_rules(id=33)
        metering_label_rules = body['metering_label_rules']
        self.assertEmpty(metering_label_rules)

    @decorators.idempotent_id('f4d547cd-3aee-408f-bf36-454f8825e045')
    def test_create_delete_metering_label_rule_with_filters(self):
        """Verifies creating and deleting metering label rule with filters"""
        # Creates a rule
        remote_ip_prefix = ("10.0.1.0/24" if self._ip_version == 4
                            else "fd03::/64")
        client = self.admin_metering_label_rules_client
        body = (client.create_metering_label_rule(
                remote_ip_prefix=remote_ip_prefix,
                direction="ingress",
                metering_label_id=self.metering_label['id']))
        metering_label_rule = body['metering_label_rule']
        self.addCleanup(self._delete_metering_label_rule,
                        metering_label_rule['id'])
        # Assert whether created rules are found in rules list or fail
        # if created rules are not found in rules list
        rules = client.list_metering_label_rules(id=metering_label_rule['id'])
        self.assertEqual(len(rules['metering_label_rules']), 1)

    @decorators.idempotent_id('b7354489-96ea-41f3-9452-bace120fb4a7')
    def test_show_metering_label_rule(self):
        """Verifies the metering details of a rule"""
        client = self.admin_metering_label_rules_client
        body = (client.show_metering_label_rule(
                self.metering_label_rule['id']))
        metering_label_rule = body['metering_label_rule']
        self.assertEqual(self.metering_label_rule['id'],
                         metering_label_rule['id'])
        self.assertEqual(self.metering_label_rule['remote_ip_prefix'],
                         metering_label_rule['remote_ip_prefix'])
        self.assertEqual(self.metering_label_rule['direction'],
                         metering_label_rule['direction'])
        self.assertEqual(self.metering_label_rule['metering_label_id'],
                         metering_label_rule['metering_label_id'])
        self.assertFalse(metering_label_rule['excluded'])


class MeteringIpV6TestJSON(MeteringTestJSON):
    _ip_version = 6



# Copyright 2013 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.image import base
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc


class ImageMembersNegativeTest(base.BaseV1ImageMembersTest):
    """Negative tests of image members"""

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('147a9536-18e3-45da-91ea-b037a028f364')
    def test_add_member_with_non_existing_image(self):
        """Add member with non existing image"""
        non_exist_image = data_utils.rand_uuid()
        self.assertRaises(lib_exc.NotFound,
                          self.image_member_client.create_image_member,
                          non_exist_image, self.alt_tenant_id)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('e1559f05-b667-4f1b-a7af-518b52dc0c0f')
    def test_delete_member_with_non_existing_image(self):
        """Delete member with non existing image"""
        non_exist_image = data_utils.rand_uuid()
        self.assertRaises(lib_exc.NotFound,
                          self.image_member_client.delete_image_member,
                          non_exist_image, self.alt_tenant_id)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('f5720333-dd69-4194-bb76-d2f048addd56')
    def test_delete_member_with_non_existing_tenant(self):
        """Delete member from image with non existing tenant"""
        image_id = self._create_image()
        non_exist_tenant = data_utils.rand_uuid_hex()
        self.assertRaises(lib_exc.NotFound,
                          self.image_member_client.delete_image_member,
                          image_id, non_exist_tenant)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('f25f89e4-0b6c-453b-a853-1f80b9d7ef26')
    def test_get_image_without_membership(self):
        """Get image without membership

        Image is hidden from another tenants.
        """
        image_id = self._create_image()
        self.assertRaises(lib_exc.NotFound,
                          self.alt_img_cli.show_image,
                          image_id)



# Copyright 2016 NEC Corporation.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib.services.identity.v2 import roles_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestRolesClient(base.BaseServiceTest):
    FAKE_ROLE_INFO = {
        "role": {
            "id": "1",
            "name": "test",
            "description": "test_description"
        }
    }

    FAKE_LIST_ROLES = {
        "roles": [
            {
                "id": "1",
                "name": "test",
                "description": "test_description"
            },
            {
                "id": "2",
                "name": "test2",
                "description": "test2_description"
            }
        ]
    }

    def setUp(self):
        super(TestRolesClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = roles_client.RolesClient(fake_auth,
                                               'identity', 'regionOne')

    def _test_create_role(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_role,
            'tempest.lib.common.rest_client.RestClient.post',
            self.FAKE_ROLE_INFO,
            bytes_body,
            id="1",
            name="test",
            description="test_description")

    def _test_show_role(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_role,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_ROLE_INFO,
            bytes_body,
            role_id_or_name="1")

    def _test_list_roles(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_roles,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_LIST_ROLES,
            bytes_body)

    def _test_create_user_role_on_project(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_user_role_on_project,
            'tempest.lib.common.rest_client.RestClient.put',
            self.FAKE_ROLE_INFO,
            bytes_body,
            tenant_id="b344506af7644f6794d9cb316600b020",
            user_id="123",
            role_id="1234",
            status=200)

    def _test_list_user_roles_on_project(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_user_roles_on_project,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_LIST_ROLES,
            bytes_body,
            tenant_id="b344506af7644f6794d9cb316600b020",
            user_id="123")

    def test_create_role_with_str_body(self):
        self._test_create_role()

    def test_create_role_with_bytes_body(self):
        self._test_create_role(bytes_body=True)

    def test_show_role_with_str_body(self):
        self._test_show_role()

    def test_show_role_with_bytes_body(self):
        self._test_show_role(bytes_body=True)

    def test_list_roles_with_str_body(self):
        self._test_list_roles()

    def test_list_roles_with_bytes_body(self):
        self._test_list_roles(bytes_body=True)

    def test_delete_role(self):
        self.check_service_client_function(
            self.client.delete_role,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            role_id="1",
            status=204)

    def test_create_user_role_on_project_with_str_body(self):
        self._test_create_user_role_on_project()

    def test_create_user_role_on_project_with_bytes_body(self):
        self._test_create_user_role_on_project(bytes_body=True)

    def test_list_user_roles_on_project_with_str_body(self):
        self._test_list_user_roles_on_project()

    def test_list_user_roles_on_project_with_bytes_body(self):
        self._test_list_user_roles_on_project(bytes_body=True)

    def test_delete_role_from_user_on_project(self):
        self.check_service_client_function(
            self.client.delete_role_from_user_on_project,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            tenant_id="b344506af7644f6794d9cb316600b020",
            user_id="123",
            role_id="1234",
            status=204)



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.compute import fixed_ips_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestFixedIPsClient(base.BaseServiceTest):
    FIXED_IP_INFO = {"fixed_ip": {"address": "10.0.0.1",
                                  "cidr": "10.11.12.0/24",
                                  "host": "localhost",
                                  "hostname": "OpenStack"}}

    def setUp(self):
        super(TestFixedIPsClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.fixedIPsClient = (fixed_ips_client.
                               FixedIPsClient
                               (fake_auth, 'compute',
                                'regionOne'))

    def _test_show_fixed_ip(self, bytes_body=False):
        self.check_service_client_function(
            self.fixedIPsClient.show_fixed_ip,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FIXED_IP_INFO, bytes_body,
            status=200, fixed_ip='Identifier')

    def test_show_fixed_ip_with_str_body(self):
        self._test_show_fixed_ip()

    def test_show_fixed_ip_with_bytes_body(self):
        self._test_show_fixed_ip(True)

    def _test_reserve_fixed_ip(self, bytes_body=False):
        self.check_service_client_function(
            self.fixedIPsClient.reserve_fixed_ip,
            'tempest.lib.common.rest_client.RestClient.post',
            {}, bytes_body,
            status=202, fixed_ip='Identifier')

    def test_reserve_fixed_ip_with_str_body(self):
        self._test_reserve_fixed_ip()

    def test_reserve_fixed_ip_with_bytes_body(self):
        self._test_reserve_fixed_ip(True)



# Copyright (C) 2017 Dell Inc. or its subsidiaries.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from urllib import parse as urllib

from oslo_serialization import jsonutils as json

from tempest.lib.api_schema.response.volume import groups as schema
from tempest.lib.common import rest_client
from tempest.lib import exceptions as lib_exc
from tempest.lib.services.volume import base_client


class GroupsClient(base_client.BaseClient):
    """Client class to send CRUD Volume Group API requests"""
    api_version = 'v3'

    def create_group(self, **kwargs):
        """Creates a group.

        group_type and volume_types are required parameters in kwargs.
        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/#create-group
        """
        post_body = json.dumps({'group': kwargs})
        resp, body = self.post('groups', post_body)
        body = json.loads(body)
        self.validate_response(schema.create_group, resp, body)
        return rest_client.ResponseBody(resp, body)

    def delete_group(self, group_id, delete_volumes=True):
        """Deletes a group.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/#delete-group
        """
        post_body = {'delete-volumes': delete_volumes}
        post_body = json.dumps({'delete': post_body})
        resp, body = self.post('groups/%s/action' % group_id,
                               post_body)
        self.validate_response(schema.delete_group, resp, body)
        return rest_client.ResponseBody(resp, body)

    def show_group(self, group_id):
        """Returns the details of a single group.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/#show-group-details
        """
        url = "groups/%s" % str(group_id)
        resp, body = self.get(url)
        body = json.loads(body)
        self.validate_response(schema.show_group, resp, body)
        return rest_client.ResponseBody(resp, body)

    def list_groups(self, detail=False, **params):
        """Lists information for all the tenant's groups.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/#list-groups
        https://docs.openstack.org/api-ref/block-storage/v3/#list-groups-with-details
        """
        url = "groups"
        schema_list_groups = schema.list_groups_no_detail
        if detail:
            url += "/detail"
            schema_list_groups = schema.list_groups_with_detail
        if params:
            url += '?%s' % urllib.urlencode(params)
        resp, body = self.get(url)
        body = json.loads(body)
        self.validate_response(schema_list_groups, resp, body)
        return rest_client.ResponseBody(resp, body)

    def create_group_from_source(self, **kwargs):
        """Creates a group from source.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/#create-group-from-source
        """
        post_body = json.dumps({'create-from-src': kwargs})
        resp, body = self.post('groups/action', post_body)
        body = json.loads(body)
        self.validate_response(schema.create_group_from_source, resp, body)
        return rest_client.ResponseBody(resp, body)

    def update_group(self, group_id, **kwargs):
        """Updates the specified group.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/#update-group
        """
        put_body = json.dumps({'group': kwargs})
        resp, body = self.put('groups/%s' % group_id, put_body)
        self.validate_response(schema.update_group, resp, body)
        return rest_client.ResponseBody(resp, body)

    def reset_group_status(self, group_id, status_to_set):
        """Resets group status.

        For more information, please refer to the official API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/#reset-group-status
        """
        post_body = json.dumps({'reset_status': {'status': status_to_set}})
        resp, body = self.post('groups/%s/action' % group_id, post_body)
        self.validate_response(schema.reset_group_status, resp, body)
        return rest_client.ResponseBody(resp, body)

    def is_resource_deleted(self, id):
        try:
            self.show_group(id)
        except lib_exc.NotFound:
            return True
        return False

    @property
    def resource_type(self):
        """Returns the primary type of resource this client works with."""
        return 'group'



# Copyright 2014 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils as json

from tempest.lib.common import rest_client
from tempest.lib import exceptions as lib_exc


class BackupsClient(rest_client.RestClient):
    """Volume V1 Backups client"""
    api_version = "v1"

    def create_backup(self, **kwargs):
        """Creates a backup of volume.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v2/#create-backup
        """
        post_body = json.dumps({'backup': kwargs})
        resp, body = self.post('backups', post_body)
        body = json.loads(body)
        self.expected_success(202, resp.status)
        return rest_client.ResponseBody(resp, body)

    def restore_backup(self, backup_id, **kwargs):
        """Restore volume from backup.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v2/#restore-backup
        """
        post_body = json.dumps({'restore': kwargs})
        resp, body = self.post('backups/%s/restore' % (backup_id), post_body)
        body = json.loads(body)
        self.expected_success(202, resp.status)
        return rest_client.ResponseBody(resp, body)

    def delete_backup(self, backup_id):
        """Delete a backup of volume."""
        resp, body = self.delete('backups/%s' % backup_id)
        self.expected_success(202, resp.status)
        return rest_client.ResponseBody(resp, body)

    def show_backup(self, backup_id):
        """Returns the details of a single backup."""
        url = "backups/%s" % backup_id
        resp, body = self.get(url)
        body = json.loads(body)
        self.expected_success(200, resp.status)
        return rest_client.ResponseBody(resp, body)

    def list_backups(self, detail=False):
        """Information for all the tenant's backups."""
        url = "backups"
        if detail:
            url += "/detail"
        resp, body = self.get(url)
        body = json.loads(body)
        self.expected_success(200, resp.status)
        return rest_client.ResponseBody(resp, body)

    def export_backup(self, backup_id):
        """Export backup metadata record."""
        url = "backups/%s/export_record" % backup_id
        resp, body = self.get(url)
        body = json.loads(body)
        self.expected_success(200, resp.status)
        return rest_client.ResponseBody(resp, body)

    def import_backup(self, **kwargs):
        """Import backup metadata record."""
        # TODO(linanbj): Current api-site doesn't contain this API description.
        # After fixing the api-site, we need to fix here also for putting the
        # link to api-site.
        post_body = json.dumps({'backup-record': kwargs})
        resp, body = self.post("backups/import_record", post_body)
        body = json.loads(body)
        self.expected_success(201, resp.status)
        return rest_client.ResponseBody(resp, body)

    def reset_backup_status(self, backup_id, status):
        """Reset the specified backup's status."""
        post_body = json.dumps({'os-reset_status': {"status": status}})
        resp, body = self.post('backups/%s/action' % backup_id, post_body)
        self.expected_success(202, resp.status)
        return rest_client.ResponseBody(resp, body)

    def is_resource_deleted(self, id):
        try:
            self.show_backup(id)
        except lib_exc.NotFound:
            return True
        return False

    @property
    def resource_type(self):
        """Returns the primary type of resource this client works with."""
        return 'backup'



# Copyright (c) 2019 Ericsson
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from tempest.common import utils
from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from tempest.scenario import manager


CONF = config.CONF


class MinBwAllocationPlacementTest(manager.NetworkScenarioTest):
    credentials = ['primary', 'admin']
    required_extensions = ['port-resource-request',
                           'qos',
                           'qos-bw-minimum-ingress']
    # The feature QoS minimum bandwidth allocation in Placement API depends on
    # Granular resource requests to GET /allocation_candidates and Support
    # allocation candidates with nested resource providers features in
    # Placement (see: https://specs.openstack.org/openstack/nova-specs/specs/
    # stein/approved/bandwidth-resource-provider.html#rest-api-impact) and this
    # means that the minimum placement microversion is 1.29
    placement_min_microversion = '1.29'
    placement_max_microversion = 'latest'

    # Nova rejects to boot VM with port which has resource_request field, below
    # microversion 2.72
    compute_min_microversion = '2.72'
    compute_max_microversion = 'latest'

    INGRESS_RESOURCE_CLASS = "NET_BW_IGR_KILOBIT_PER_SEC"
    INGRESS_DIRECTION = 'ingress'

    SMALLEST_POSSIBLE_BW = 1
    # For any realistic inventory value (that is inventory != MAX_INT) an
    # allocation candidate request of MAX_INT is expected to be rejected, see:
    # https://github.com/openstack/placement/blob/master/placement/
    # db/constants.py#L16
    PLACEMENT_MAX_INT = 0x7FFFFFFF
    BANDWIDTH_1 = 1000
    BANDWIDTH_2 = 2000

    @classmethod
    def setup_clients(cls):
        super(MinBwAllocationPlacementTest, cls).setup_clients()
        cls.placement_client = cls.os_admin.placement_client
        cls.networks_client = cls.os_admin.networks_client
        cls.subnets_client = cls.os_admin.subnets_client
        cls.ports_client = cls.os_primary.ports_client
        cls.routers_client = cls.os_adm.routers_client
        cls.qos_client = cls.os_admin.qos_client
        cls.qos_min_bw_client = cls.os_admin.qos_min_bw_client
        cls.flavors_client = cls.os_adm.flavors_client
        cls.servers_client = cls.os_adm.servers_client

    @classmethod
    def skip_checks(cls):
        super(MinBwAllocationPlacementTest, cls).skip_checks()
        if not CONF.network_feature_enabled.qos_placement_physnet:
            msg = "Skipped as no physnet is available in config for " \
                  "placement based QoS allocation."
            raise cls.skipException(msg)

    def setUp(self):
        super(MinBwAllocationPlacementTest, self).setUp()
        self._check_if_allocation_is_possible()

    def _create_policy_and_min_bw_rule(self, name_prefix, min_kbps):
        policy = self.qos_client.create_qos_policy(
            name=data_utils.rand_name(name_prefix),
            shared=True)['policy']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.qos_client.delete_qos_policy, policy['id'])
        rule = self.qos_min_bw_client.create_minimum_bandwidth_rule(
            policy['id'],
            **{
                'min_kbps': min_kbps,
                'direction': self.INGRESS_DIRECTION
            })['minimum_bandwidth_rule']
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.qos_min_bw_client.delete_minimum_bandwidth_rule, policy['id'],
            rule['id'])

        return policy

    def _create_qos_basic_policies(self):
        self.qos_policy_valid = self._create_policy_and_min_bw_rule(
            name_prefix='test_policy_valid',
            min_kbps=self.SMALLEST_POSSIBLE_BW)
        self.qos_policy_not_valid = self._create_policy_and_min_bw_rule(
            name_prefix='test_policy_not_valid',
            min_kbps=self.PLACEMENT_MAX_INT)

    def _create_qos_policies_from_life(self):
        # For tempest-slow the max bandwidth configured is 1000000,
        # https://opendev.org/openstack/tempest/src/branch/master/
        # .zuul.yaml#L416-L420
        self.qos_policy_1 = self._create_policy_and_min_bw_rule(
            name_prefix='test_policy_1',
            min_kbps=self.BANDWIDTH_1
        )
        self.qos_policy_2 = self._create_policy_and_min_bw_rule(
            name_prefix='test_policy_2',
            min_kbps=self.BANDWIDTH_2
        )

    def _create_network_and_qos_policies(self, policy_method):
        physnet_name = CONF.network_feature_enabled.qos_placement_physnet
        base_segm = \
            CONF.network_feature_enabled.provider_net_base_segmentation_id

        self.prov_network, _, _ = self.setup_network_subnet_with_router(
            networks_client=self.networks_client,
            routers_client=self.routers_client,
            subnets_client=self.subnets_client,
            **{
                'shared': True,
                'provider:network_type': 'vlan',
                'provider:physical_network': physnet_name,
                'provider:segmentation_id': base_segm
            })

        policy_method()

    def _check_if_allocation_is_possible(self):
        alloc_candidates = self.placement_client.list_allocation_candidates(
            resources1='%s:%s' % (self.INGRESS_RESOURCE_CLASS,
                                  self.SMALLEST_POSSIBLE_BW))
        if len(alloc_candidates['provider_summaries']) == 0:
            # Skip if the backend does not support QoS minimum bandwidth
            # allocation in Placement API
            raise self.skipException(
                'No allocation candidates are available for %s:%s' %
                (self.INGRESS_RESOURCE_CLASS, self.SMALLEST_POSSIBLE_BW))

        # Just to be sure check with impossible high (placement max_int),
        # allocation
        alloc_candidates = self.placement_client.list_allocation_candidates(
            resources1='%s:%s' % (self.INGRESS_RESOURCE_CLASS,
                                  self.PLACEMENT_MAX_INT))
        if len(alloc_candidates['provider_summaries']) != 0:
            self.fail('For %s:%s there should be no available candidate!' %
                      (self.INGRESS_RESOURCE_CLASS, self.PLACEMENT_MAX_INT))

    def _boot_vm_with_min_bw(self, qos_policy_id, status='ACTIVE'):
        wait_until = (None if status == 'ERROR' else status)
        port = self.create_port(
            self.prov_network['id'], qos_policy_id=qos_policy_id)

        server = self.create_server(networks=[{'port': port['id']}],
                                    wait_until=wait_until)
        waiters.wait_for_server_status(
            client=self.os_primary.servers_client, server_id=server['id'],
            status=status, ready_wait=False, raise_on_error=False)
        return server, port

    def _assert_allocation_is_as_expected(self, consumer, port_ids,
                                          min_kbps=SMALLEST_POSSIBLE_BW):
        allocations = self.placement_client.list_allocations(
            consumer)['allocations']
        self.assertGreater(len(allocations), 0)
        bw_resource_in_alloc = False
        for rp, resources in allocations.items():
            if self.INGRESS_RESOURCE_CLASS in resources['resources']:
                self.assertEqual(
                    min_kbps,
                    resources['resources'][self.INGRESS_RESOURCE_CLASS])
                bw_resource_in_alloc = True
                allocation_rp = rp
        if min_kbps:
            self.assertTrue(bw_resource_in_alloc)

            # Check binding_profile of the port is not empty and equals with
            # the rp uuid
            for port_id in port_ids:
                port = self.os_admin.ports_client.show_port(port_id)
                self.assertEqual(
                    allocation_rp,
                    port['port']['binding:profile']['allocation'])

    @decorators.idempotent_id('78625d92-212c-400e-8695-dd51706858b8')
    @utils.services('compute', 'network')
    def test_qos_min_bw_allocation_basic(self):
        """"Basic scenario with QoS min bw allocation in placement.

        Steps:
        * Create prerequisites:
        ** VLAN type provider network with subnet.
        ** valid QoS policy with minimum bandwidth rule with min_kbps=1
        (This is a simplification to skip the checks in placement for
        detecting the resource provider tree and inventories, as if
        bandwidth resource is available 1 kbs will be available).
        ** invalid QoS policy with minimum bandwidth rule with
        min_kbs=max integer from placement (this is a simplification again
        to avoid detection of RP tress and inventories, as placement will
        reject such big allocation).
        * Create port with valid QoS policy, and boot VM with that, it should
        pass.
        * Create port with invalid QoS policy, and try to boot VM with that,
        it should fail.
        """
        self._create_network_and_qos_policies(self._create_qos_basic_policies)
        server1, valid_port = self._boot_vm_with_min_bw(
            qos_policy_id=self.qos_policy_valid['id'])
        self._assert_allocation_is_as_expected(server1['id'],
                                               [valid_port['id']])

        server2, not_valid_port = self._boot_vm_with_min_bw(
            self.qos_policy_not_valid['id'], status='ERROR')
        allocations = self.placement_client.list_allocations(server2['id'])

        self.assertEqual(0, len(allocations['allocations']))
        server2 = self.servers_client.show_server(server2['id'])
        self.assertIn('fault', server2['server'])
        self.assertIn('No valid host', server2['server']['fault']['message'])
        # Check that binding_profile of the port is empty
        port = self.os_admin.ports_client.show_port(not_valid_port['id'])
        self.assertEqual(0, len(port['port']['binding:profile']))

    @decorators.idempotent_id('8a98150c-a506-49a5-96c6-73a5e7b04ada')
    @testtools.skipUnless(CONF.compute_feature_enabled.cold_migration,
                          'Cold migration is not available.')
    @testtools.skipUnless(CONF.compute.min_compute_nodes > 1,
                          'Less than 2 compute nodes, skipping multinode '
                          'tests.')
    @utils.services('compute', 'network')
    def test_migrate_with_qos_min_bw_allocation(self):
        """Scenario to migrate VM with QoS min bw allocation in placement

        Boot a VM like in test_qos_min_bw_allocation_basic, do the same
        checks, and
        * migrate the server
        * confirm the resize, if the VM state is VERIFY_RESIZE
        * If the VM goes to ACTIVE state check that allocations are as
        expected.
        """
        self._create_network_and_qos_policies(self._create_qos_basic_policies)
        server, valid_port = self._boot_vm_with_min_bw(
            qos_policy_id=self.qos_policy_valid['id'])
        self._assert_allocation_is_as_expected(server['id'],
                                               [valid_port['id']])

        self.servers_client.migrate_server(server_id=server['id'])
        waiters.wait_for_server_status(
            client=self.os_primary.servers_client, server_id=server['id'],
            status='VERIFY_RESIZE', ready_wait=False, raise_on_error=False)

        # TODO(lajoskatona): Check that the allocations are ok for the
        #  migration?
        self._assert_allocation_is_as_expected(server['id'],
                                               [valid_port['id']])

        self.servers_client.confirm_resize_server(server_id=server['id'])
        waiters.wait_for_server_status(
            client=self.os_primary.servers_client, server_id=server['id'],
            status='ACTIVE', ready_wait=False, raise_on_error=True)
        self._assert_allocation_is_as_expected(server['id'],
                                               [valid_port['id']])

    @decorators.idempotent_id('c29e7fd3-035d-4993-880f-70819847683f')
    @testtools.skipUnless(CONF.compute_feature_enabled.resize,
                          'Resize not available.')
    @utils.services('compute', 'network')
    def test_resize_with_qos_min_bw_allocation(self):
        """Scenario to resize VM with QoS min bw allocation in placement.

        Boot a VM like in test_qos_min_bw_allocation_basic, do the same
        checks, and
        * resize the server with new flavor
        * confirm the resize, if the VM state is VERIFY_RESIZE
        * If the VM goes to ACTIVE state check that allocations are as
        expected.
        """
        self._create_network_and_qos_policies(self._create_qos_basic_policies)
        server, valid_port = self._boot_vm_with_min_bw(
            qos_policy_id=self.qos_policy_valid['id'])
        self._assert_allocation_is_as_expected(server['id'],
                                               [valid_port['id']])

        old_flavor = self.flavors_client.show_flavor(
            CONF.compute.flavor_ref)['flavor']
        new_flavor = self.flavors_client.create_flavor(**{
            'ram': old_flavor['ram'],
            'vcpus': old_flavor['vcpus'],
            'name': old_flavor['name'] + 'extra',
            'disk': old_flavor['disk'] + 1
        })['flavor']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.flavors_client.delete_flavor, new_flavor['id'])

        self.servers_client.resize_server(
            server_id=server['id'], flavor_ref=new_flavor['id'])
        waiters.wait_for_server_status(
            client=self.os_primary.servers_client, server_id=server['id'],
            status='VERIFY_RESIZE', ready_wait=False, raise_on_error=False)

        # TODO(lajoskatona): Check that the allocations are ok for the
        #  migration?
        self._assert_allocation_is_as_expected(server['id'],
                                               [valid_port['id']])

        self.servers_client.confirm_resize_server(server_id=server['id'])
        waiters.wait_for_server_status(
            client=self.os_primary.servers_client, server_id=server['id'],
            status='ACTIVE', ready_wait=False, raise_on_error=True)
        self._assert_allocation_is_as_expected(server['id'],
                                               [valid_port['id']])

    @decorators.idempotent_id('79fdaa1c-df62-4738-a0f0-1cff9dc415f6')
    @utils.services('compute', 'network')
    def test_qos_min_bw_allocation_update_policy(self):
        """Test the update of QoS policy on bound port

        Related RFE in neutron: #1882804
        The scenario is the following:
        * Have a port with QoS policy and minimum bandwidth rule.
        * Boot a VM with the port.
        * Update the port with a new policy with different minimum bandwidth
        values.
        * The allocation on placement side should be according to the new
        rules.
        """
        if not utils.is_network_feature_enabled('update_port_qos'):
            raise self.skipException("update_port_qos feature is not enabled")

        self._create_network_and_qos_policies(
            self._create_qos_policies_from_life)

        port = self.create_port(
            self.prov_network['id'], qos_policy_id=self.qos_policy_1['id'])

        server1 = self.create_server(
            networks=[{'port': port['id']}])

        self._assert_allocation_is_as_expected(server1['id'], [port['id']],
                                               self.BANDWIDTH_1)

        self.ports_client.update_port(
            port['id'],
            **{'qos_policy_id': self.qos_policy_2['id']})
        self._assert_allocation_is_as_expected(server1['id'], [port['id']],
                                               self.BANDWIDTH_2)

        # I changed my mind
        self.ports_client.update_port(
            port['id'],
            **{'qos_policy_id': self.qos_policy_1['id']})
        self._assert_allocation_is_as_expected(server1['id'], [port['id']],
                                               self.BANDWIDTH_1)

        # bad request....
        self.qos_policy_not_valid = self._create_policy_and_min_bw_rule(
            name_prefix='test_policy_not_valid',
            min_kbps=self.PLACEMENT_MAX_INT)
        port_orig = self.ports_client.show_port(port['id'])['port']
        self.assertRaises(
            lib_exc.Conflict,
            self.ports_client.update_port,
            port['id'], **{'qos_policy_id': self.qos_policy_not_valid['id']})
        self._assert_allocation_is_as_expected(server1['id'], [port['id']],
                                               self.BANDWIDTH_1)

        port_upd = self.ports_client.show_port(port['id'])['port']
        self.assertEqual(port_orig['qos_policy_id'],
                         port_upd['qos_policy_id'])
        self.assertEqual(self.qos_policy_1['id'], port_upd['qos_policy_id'])

    @decorators.idempotent_id('9cfc3bb8-f433-4c91-87b6-747cadc8958a')
    @utils.services('compute', 'network')
    def test_qos_min_bw_allocation_update_policy_from_zero(self):
        """Test port without QoS policy to have QoS policy

        This scenario checks if updating a port without QoS policy to
        have QoS policy with minimum_bandwidth rule succeeds only on
        controlplane, but placement allocation remains 0.
        """
        if not utils.is_network_feature_enabled('update_port_qos'):
            raise self.skipException("update_port_qos feature is not enabled")

        self._create_network_and_qos_policies(
            self._create_qos_policies_from_life)

        port = self.create_port(self.prov_network['id'])

        server1 = self.create_server(
            networks=[{'port': port['id']}])

        self._assert_allocation_is_as_expected(server1['id'], [port['id']], 0)

        self.ports_client.update_port(
            port['id'], **{'qos_policy_id': self.qos_policy_2['id']})
        self._assert_allocation_is_as_expected(server1['id'], [port['id']], 0)

    @decorators.idempotent_id('a9725a70-1d28-4e3b-ae0e-450abc235962')
    @utils.services('compute', 'network')
    def test_qos_min_bw_allocation_update_policy_to_zero(self):
        """Test port with QoS policy to remove QoS policy

        In this scenario port with QoS minimum_bandwidth rule update to
        remove QoS policy results in 0 placement allocation.
        """
        if not utils.is_network_feature_enabled('update_port_qos'):
            raise self.skipException("update_port_qos feature is not enabled")

        self._create_network_and_qos_policies(
            self._create_qos_policies_from_life)

        port = self.create_port(
            self.prov_network['id'], qos_policy_id=self.qos_policy_1['id'])

        server1 = self.create_server(
            networks=[{'port': port['id']}])
        self._assert_allocation_is_as_expected(server1['id'], [port['id']],
                                               self.BANDWIDTH_1)

        self.ports_client.update_port(
            port['id'],
            **{'qos_policy_id': None})
        self._assert_allocation_is_as_expected(server1['id'], [port['id']], 0)

    @decorators.idempotent_id('756ced7f-6f1a-43e7-a851-2fcfc16f3dd7')
    @utils.services('compute', 'network')
    def test_qos_min_bw_allocation_update_with_multiple_ports(self):
        if not utils.is_network_feature_enabled('update_port_qos'):
            raise self.skipException("update_port_qos feature is not enabled")

        self._create_network_and_qos_policies(
            self._create_qos_policies_from_life)

        port1 = self.create_port(
            self.prov_network['id'], qos_policy_id=self.qos_policy_1['id'])
        port2 = self.create_port(
            self.prov_network['id'], qos_policy_id=self.qos_policy_2['id'])

        server1 = self.create_server(
            networks=[{'port': port1['id']}, {'port': port2['id']}])
        self._assert_allocation_is_as_expected(
            server1['id'], [port1['id'], port2['id']],
            self.BANDWIDTH_1 + self.BANDWIDTH_2)

        self.ports_client.update_port(
            port1['id'],
            **{'qos_policy_id': self.qos_policy_2['id']})
        self._assert_allocation_is_as_expected(
            server1['id'], [port1['id'], port2['id']],
            2 * self.BANDWIDTH_2)

    @decorators.idempotent_id('0805779e-e03c-44fb-900f-ce97a790653b')
    @utils.services('compute', 'network')
    def test_empty_update(self):
        if not utils.is_network_feature_enabled('update_port_qos'):
            raise self.skipException("update_port_qos feature is not enabled")

        self._create_network_and_qos_policies(
            self._create_qos_policies_from_life)

        port = self.create_port(
            self.prov_network['id'], qos_policy_id=self.qos_policy_1['id'])

        server1 = self.create_server(
            networks=[{'port': port['id']}])
        self._assert_allocation_is_as_expected(server1['id'], [port['id']],
                                               self.BANDWIDTH_1)
        self.ports_client.update_port(
            port['id'],
            **{'description': 'foo'})
        self._assert_allocation_is_as_expected(server1['id'], [port['id']],
                                               self.BANDWIDTH_1)



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.tests.lib import fake_auth_provider

from tempest.lib.services.compute import floating_ips_bulk_client
from tempest.tests.lib.services import base


class TestFloatingIPsBulkClient(base.BaseServiceTest):

    FAKE_FIP_BULK_LIST = {"floating_ip_info": [{
        "address": "10.10.10.1",
        "instance_uuid": None,
        "fixed_ip": None,
        "interface": "eth0",
        "pool": "nova",
        "project_id": None
        },
        {
        "address": "10.10.10.2",
        "instance_uuid": None,
        "fixed_ip": None,
        "interface": "eth0",
        "pool": "nova",
        "project_id": None
        }]}

    def setUp(self):
        super(TestFloatingIPsBulkClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = floating_ips_bulk_client.FloatingIPsBulkClient(
            fake_auth, 'compute', 'regionOne')

    def _test_list_floating_ips_bulk(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_floating_ips_bulk,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_FIP_BULK_LIST,
            to_utf=bytes_body)

    def _test_create_floating_ips_bulk(self, bytes_body=False):
        fake_fip_create_data = {"floating_ips_bulk_create": {
            "ip_range": "192.168.1.0/24", "pool": "nova", "interface": "eth0"}}
        self.check_service_client_function(
            self.client.create_floating_ips_bulk,
            'tempest.lib.common.rest_client.RestClient.post',
            fake_fip_create_data,
            to_utf=bytes_body,
            ip_range="192.168.1.0/24", pool="nova", interface="eth0")

    def _test_delete_floating_ips_bulk(self, bytes_body=False):
        fake_fip_delete_data = {"floating_ips_bulk_delete": "192.168.1.0/24"}
        self.check_service_client_function(
            self.client.delete_floating_ips_bulk,
            'tempest.lib.common.rest_client.RestClient.put',
            fake_fip_delete_data,
            to_utf=bytes_body,
            ip_range="192.168.1.0/24")

    def test_list_floating_ips_bulk_with_str_body(self):
        self._test_list_floating_ips_bulk()

    def test_list_floating_ips_bulk_with_bytes_body(self):
        self._test_list_floating_ips_bulk(True)

    def test_create_floating_ips_bulk_with_str_body(self):
        self._test_create_floating_ips_bulk()

    def test_create_floating_ips_bulk_with_bytes_body(self):
        self._test_create_floating_ips_bulk(True)

    def test_delete_floating_ips_bulk_with_str_body(self):
        self._test_delete_floating_ips_bulk()

    def test_delete_floating_ips_bulk_with_bytes_body(self):
        self._test_delete_floating_ips_bulk(True)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import log as logging
from oslo_serialization import jsonutils as json
import testtools

from tempest.common import utils
from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.scenario import manager

CONF = config.CONF
LOG = logging.getLogger(__name__)


class TestVolumeBootPattern(manager.EncryptionScenarioTest):

    # Boot from volume scenario is quite slow, and needs extra
    # breathing room to get through deletes in the time allotted.
    TIMEOUT_SCALING_FACTOR = 2

    def _delete_server(self, server):
        self.servers_client.delete_server(server['id'])
        waiters.wait_for_server_termination(self.servers_client, server['id'])

    def _delete_snapshot(self, snapshot_id):
        self.snapshots_client.delete_snapshot(snapshot_id)
        self.snapshots_client.wait_for_resource_deletion(snapshot_id)

    @decorators.idempotent_id('557cd2c2-4eb8-4dce-98be-f86765ff311b')
    @decorators.attr(type='slow')
    # Note: This test is being skipped based on 'public_network_id'.
    # It is being used in create_floating_ip() method which gets called
    # from get_server_ip() method
    @testtools.skipUnless(CONF.network.public_network_id,
                          'The public_network_id option must be specified.')
    @testtools.skipUnless(CONF.volume_feature_enabled.snapshot,
                          'Cinder volume snapshots are disabled')
    @utils.services('compute', 'volume', 'image')
    def test_volume_boot_pattern(self):
        """This test case attempts to reproduce the following steps:

        * Create in Cinder some bootable volume importing a Glance image
        * Boot an instance from the bootable volume
        * Write content to the volume
        * Delete an instance and Boot a new instance from the volume
        * Check written content in the instance
        * Create a volume snapshot while the instance is running
        * Boot an additional instance from the new snapshot based volume
        * Check written content in the instance booted from snapshot
        """

        LOG.info("Creating keypair and security group")
        keypair = self.create_keypair()
        security_group = self.create_security_group()

        # create an instance from volume
        LOG.info("Booting instance 1 from volume")
        volume_origin = self.create_volume_from_image()
        instance_1st = self.boot_instance_from_resource(
            source_id=volume_origin['id'],
            source_type='volume',
            keypair=keypair,
            security_group=security_group)
        LOG.info("Booted first instance: %s", instance_1st)

        # write content to volume on instance
        LOG.info("Setting timestamp in instance %s", instance_1st)
        ip_instance_1st = self.get_server_ip(instance_1st)
        timestamp = self.create_timestamp(ip_instance_1st,
                                          private_key=keypair['private_key'],
                                          server=instance_1st)

        # delete instance
        LOG.info("Deleting first instance: %s", instance_1st)
        self._delete_server(instance_1st)

        # create a 2nd instance from volume
        instance_2nd = self.boot_instance_from_resource(
            source_id=volume_origin['id'],
            source_type='volume',
            keypair=keypair,
            security_group=security_group)
        LOG.info("Booted second instance %s", instance_2nd)

        # check the content of written file
        LOG.info("Getting timestamp in instance %s", instance_2nd)
        ip_instance_2nd = self.get_server_ip(instance_2nd)
        timestamp2 = self.get_timestamp(ip_instance_2nd,
                                        private_key=keypair['private_key'],
                                        server=instance_2nd)
        self.assertEqual(timestamp, timestamp2)

        # snapshot a volume
        LOG.info("Creating snapshot from volume: %s", volume_origin['id'])
        snapshot = self.create_volume_snapshot(volume_origin['id'], force=True)

        # create a 3rd instance from snapshot
        LOG.info("Creating third instance from snapshot: %s", snapshot['id'])
        volume = self.create_volume(snapshot_id=snapshot['id'],
                                    size=snapshot['size'])
        LOG.info("Booting third instance from snapshot")
        server_from_snapshot = (
            self.boot_instance_from_resource(source_id=volume['id'],
                                             source_type='volume',
                                             keypair=keypair,
                                             security_group=security_group))
        LOG.info("Booted third instance %s", server_from_snapshot)

        # check the content of written file
        LOG.info("Logging into third instance to get timestamp: %s",
                 server_from_snapshot)
        server_from_snapshot_ip = self.get_server_ip(server_from_snapshot)
        timestamp3 = self.get_timestamp(server_from_snapshot_ip,
                                        private_key=keypair['private_key'],
                                        server=server_from_snapshot)
        self.assertEqual(timestamp, timestamp3)

    @decorators.idempotent_id('05795fb2-b2a7-4c9f-8fac-ff25aedb1489')
    @decorators.attr(type='slow')
    @testtools.skipUnless(CONF.volume_feature_enabled.snapshot,
                          'Cinder volume snapshots are disabled')
    @utils.services('compute', 'image', 'volume')
    def test_create_server_from_volume_snapshot(self):
        # Create a volume from an image
        boot_volume = self.create_volume_from_image()

        # Create a snapshot
        boot_snapshot = self.create_volume_snapshot(boot_volume['id'])

        # Create a server from a volume snapshot
        server = self.boot_instance_from_resource(
            source_id=boot_snapshot['id'],
            source_type='snapshot',
            delete_on_termination=True)

        server_info = self.servers_client.show_server(server['id'])['server']

        # The created volume when creating a server from a snapshot
        created_volume = server_info['os-extended-volumes:volumes_attached']

        self.assertNotEmpty(created_volume, "No volume attachment found.")

        created_volume_info = self.volumes_client.show_volume(
            created_volume[0]['id'])['volume']

        # Verify the server was created from the snapshot
        self.assertEqual(
            boot_volume['volume_image_metadata']['image_id'],
            created_volume_info['volume_image_metadata']['image_id'])
        self.assertEqual(boot_snapshot['id'],
                         created_volume_info['snapshot_id'])
        self.assertEqual(server['id'],
                         created_volume_info['attachments'][0]['server_id'])
        self.assertEqual(created_volume[0]['id'],
                         created_volume_info['attachments'][0]['volume_id'])

        # Delete the server and wait
        self._delete_server(server)

        # Assert that the underlying volume is gone before class tearDown
        # to prevent snapshot deletion from failing
        self.volumes_client.wait_for_resource_deletion(created_volume[0]['id'])

    @decorators.idempotent_id('36c34c67-7b54-4b59-b188-02a2f458a63b')
    @testtools.skipUnless(CONF.volume_feature_enabled.snapshot,
                          'Cinder volume snapshots are disabled')
    @utils.services('compute', 'volume', 'image')
    def test_image_defined_boot_from_volume(self):
        # create an instance from image-backed volume
        volume_origin = self.create_volume_from_image()
        name = data_utils.rand_name(self.__class__.__name__ +
                                    '-volume-backed-server')
        instance1 = self.boot_instance_from_resource(
            source_id=volume_origin['id'],
            source_type='volume',
            delete_on_termination=True,
            name=name)
        # Create a snapshot image from the volume-backed server.
        # The compute service will have the block service create a snapshot of
        # the root volume and store its metadata in the image.
        image = self.create_server_snapshot(instance1)

        # Create a server from the image snapshot which has an
        # "image-defined block device mapping (BDM)" in it, i.e. the metadata
        # about the volume snapshot. The compute service will use this to
        # create a volume from the volume snapshot and use that as the root
        # disk for the server.
        name = data_utils.rand_name(self.__class__.__name__ +
                                    '-image-snapshot-server')
        instance2 = self.create_server(image_id=image['id'], name=name)

        # Verify the server was created from the image-defined BDM.
        volume_attachments = instance2['os-extended-volumes:volumes_attached']
        self.assertEqual(1, len(volume_attachments),
                         "No volume attachment found.")
        created_volume = self.volumes_client.show_volume(
            volume_attachments[0]['id'])['volume']
        # Assert that the volume service also shows the server attachment.
        self.assertEqual(1, len(created_volume['attachments']),
                         "No server attachment found for volume: %s" %
                         created_volume)
        self.assertEqual(instance2['id'],
                         created_volume['attachments'][0]['server_id'])
        self.assertEqual(volume_attachments[0]['id'],
                         created_volume['attachments'][0]['volume_id'])
        self.assertEqual(
            volume_origin['volume_image_metadata']['image_id'],
            created_volume['volume_image_metadata']['image_id'])

        # Delete the second server which should also delete the second volume
        # created from the volume snapshot.
        self._delete_server(instance2)

        # Assert that the underlying volume is gone.
        self.volumes_client.wait_for_resource_deletion(created_volume['id'])

        # Delete the volume snapshot. We must do this before deleting the first
        # server created in this test because the snapshot depends on the first
        # instance's underlying volume (volume_origin).
        # In glance v2, the image properties are flattened and in glance v1,
        # the image properties are under the 'properties' key.
        bdms = image.get('block_device_mapping')
        if not bdms:
            bdms = image['properties']['block_device_mapping']
        bdms = json.loads(bdms)
        snapshot_id = bdms[0]['snapshot_id']
        self._delete_snapshot(snapshot_id)

        # Now, delete the first server which will also delete the first
        # image-backed volume.
        self._delete_server(instance1)

        # Assert that the underlying volume is gone.
        self.volumes_client.wait_for_resource_deletion(volume_origin['id'])

    @decorators.idempotent_id('cb78919a-e553-4bab-b73b-10cf4d2eb125')
    @testtools.skipUnless(CONF.compute_feature_enabled.attach_encrypted_volume,
                          'Encrypted volume attach is not supported')
    @utils.services('compute', 'volume')
    def test_boot_server_from_encrypted_volume_luks(self):
        # Create an encrypted volume
        volume = self.create_encrypted_volume('luks',
                                              volume_type='luks')

        self.volumes_client.set_bootable_volume(volume['id'], bootable=True)

        # Boot a server from the encrypted volume
        server = self.boot_instance_from_resource(
            source_id=volume['id'],
            source_type='volume',
            delete_on_termination=False)

        server_info = self.servers_client.show_server(server['id'])['server']
        created_volume = server_info['os-extended-volumes:volumes_attached']
        self.assertEqual(volume['id'], created_volume[0]['id'])



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


import io
import select
import socket
import time
import warnings

from oslo_log import log as logging
import six

from tempest.lib import exceptions


with warnings.catch_warnings():
    warnings.simplefilter("ignore")
    import paramiko


LOG = logging.getLogger(__name__)


class Client(object):

    def __init__(self, host, username, password=None, timeout=300, pkey=None,
                 channel_timeout=10, look_for_keys=False, key_filename=None,
                 port=22, proxy_client=None):
        """SSH client.

        Many of parameters are just passed to the underlying implementation
        as it is.  See the paramiko documentation for more details.
        http://docs.paramiko.org/en/2.1/api/client.html#paramiko.client.SSHClient.connect

        :param host: Host to login.
        :param username: SSH username.
        :param password: SSH password, or a password to unlock private key.
        :param timeout: Timeout in seconds, including retries.
            Default is 300 seconds.
        :param pkey: Private key.
        :param channel_timeout: Channel timeout in seconds, passed to the
            paramiko.  Default is 10 seconds.
        :param look_for_keys: Whether or not to search for private keys
            in ``~/.ssh``.  Default is False.
        :param key_filename: Filename for private key to use.
        :param port: SSH port number.
        :param proxy_client: Another SSH client to provide a transport
            for ssh-over-ssh.  The default is None, which means
            not to use ssh-over-ssh.
        :type proxy_client: ``tempest.lib.common.ssh.Client`` object
        """
        self.host = host
        self.username = username
        self.port = port
        self.password = password
        if isinstance(pkey, six.string_types):
            pkey = paramiko.RSAKey.from_private_key(
                io.StringIO(str(pkey)))
        self.pkey = pkey
        self.look_for_keys = look_for_keys
        self.key_filename = key_filename
        self.timeout = int(timeout)
        self.channel_timeout = float(channel_timeout)
        self.buf_size = 1024
        self.proxy_client = proxy_client
        if (self.proxy_client and self.proxy_client.host == self.host and
                self.proxy_client.port == self.port and
                self.proxy_client.username == self.username):
            raise exceptions.SSHClientProxyClientLoop(
                host=self.host, port=self.port, username=self.username)
        self._proxy_conn = None

    def _get_ssh_connection(self, sleep=1.5, backoff=1):
        """Returns an ssh connection to the specified host."""
        bsleep = sleep
        ssh = paramiko.SSHClient()
        ssh.set_missing_host_key_policy(
            paramiko.AutoAddPolicy())
        _start_time = time.time()
        if self.pkey is not None:
            LOG.info("Creating ssh connection to '%s:%d' as '%s'"
                     " with public key authentication",
                     self.host, self.port, self.username)
        else:
            LOG.info("Creating ssh connection to '%s:%d' as '%s'"
                     " with password %s",
                     self.host, self.port, self.username, str(self.password))
        attempts = 0
        while True:
            if self.proxy_client is not None:
                proxy_chan = self._get_proxy_channel()
            else:
                proxy_chan = None
            try:
                ssh.connect(self.host, port=self.port, username=self.username,
                            password=self.password,
                            look_for_keys=self.look_for_keys,
                            key_filename=self.key_filename,
                            timeout=self.channel_timeout, pkey=self.pkey,
                            sock=proxy_chan)
                LOG.info("ssh connection to %s@%s successfully created",
                         self.username, self.host)
                return ssh
            except (EOFError,
                    socket.error, socket.timeout,
                    paramiko.SSHException) as e:
                ssh.close()
                if self._is_timed_out(_start_time):
                    LOG.exception("Failed to establish authenticated ssh"
                                  " connection to %s@%s after %d attempts. "
                                  "Proxy client: %s",
                                  self.username, self.host, attempts,
                                  self._get_proxy_client_info())
                    raise exceptions.SSHTimeout(host=self.host,
                                                user=self.username,
                                                password=self.password)
                bsleep += backoff
                attempts += 1
                LOG.warning("Failed to establish authenticated ssh"
                            " connection to %s@%s (%s). Number attempts: %s."
                            " Retry after %d seconds.",
                            self.username, self.host, e, attempts, bsleep)
                time.sleep(bsleep)

    def _is_timed_out(self, start_time):
        return (time.time() - self.timeout) > start_time

    @staticmethod
    def _can_system_poll():
        return hasattr(select, 'poll')

    def exec_command(self, cmd, encoding="utf-8"):
        """Execute the specified command on the server

        Note that this method is reading whole command outputs to memory, thus
        shouldn't be used for large outputs.

        :param str cmd: Command to run at remote server.
        :param str encoding: Encoding for result from paramiko.
                             Result will not be decoded if None.
        :returns: data read from standard output of the command.
        :raises: SSHExecCommandFailed if command returns nonzero
                 status. The exception contains command status stderr content.
        :raises: TimeoutException if cmd doesn't end when timeout expires.
        """
        ssh = self._get_ssh_connection()
        transport = ssh.get_transport()
        with transport.open_session() as channel:
            channel.fileno()  # Register event pipe
            channel.exec_command(cmd)
            channel.shutdown_write()

            # If the executing host is linux-based, poll the channel
            if self._can_system_poll():
                out_data_chunks = []
                err_data_chunks = []
                poll = select.poll()
                poll.register(channel, select.POLLIN)
                start_time = time.time()

                while True:
                    ready = poll.poll(self.channel_timeout)
                    if not any(ready):
                        if not self._is_timed_out(start_time):
                            continue
                        raise exceptions.TimeoutException(
                            "Command: '{0}' executed on host '{1}'.".format(
                                cmd, self.host))
                    if not ready[0]:  # If there is nothing to read.
                        continue
                    out_chunk = err_chunk = None
                    if channel.recv_ready():
                        out_chunk = channel.recv(self.buf_size)
                        out_data_chunks += out_chunk,
                    if channel.recv_stderr_ready():
                        err_chunk = channel.recv_stderr(self.buf_size)
                        err_data_chunks += err_chunk,
                    if not err_chunk and not out_chunk:
                        break
                out_data = b''.join(out_data_chunks)
                err_data = b''.join(err_data_chunks)
            # Just read from the channels
            else:
                out_file = channel.makefile('rb', self.buf_size)
                err_file = channel.makefile_stderr('rb', self.buf_size)
                out_data = out_file.read()
                err_data = err_file.read()
            if encoding:
                out_data = out_data.decode(encoding)
                err_data = err_data.decode(encoding)

            exit_status = channel.recv_exit_status()

        ssh.close()

        if 0 != exit_status:
            raise exceptions.SSHExecCommandFailed(
                command=cmd, exit_status=exit_status,
                stderr=err_data, stdout=out_data)
        return out_data

    def test_connection_auth(self):
        """Raises an exception when we can not connect to server via ssh."""
        connection = self._get_ssh_connection()
        connection.close()

    def _get_proxy_channel(self):
        conn = self.proxy_client._get_ssh_connection()
        # Keep a reference to avoid g/c
        # https://github.com/paramiko/paramiko/issues/440
        self._proxy_conn = conn
        transport = conn.get_transport()
        chan = transport.open_session()
        cmd = 'nc %s %s' % (self.host, self.port)
        chan.exec_command(cmd)
        return chan

    def _get_proxy_client_info(self):
        if not self.proxy_client:
            return 'no proxy client'
        nested_pclient = self.proxy_client._get_proxy_client_info()
        return ('%(username)s@%(host)s:%(port)s, nested proxy client: '
                '%(nested_pclient)s' % {'username': self.proxy_client.username,
                                        'host': self.proxy_client.host,
                                        'port': self.proxy_client.port,
                                        'nested_pclient': nested_pclient})



# Copyright 2014 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import base64
from oslo_serialization import jsonutils as json

from tempest.api.volume import base
from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

CONF = config.CONF


class VolumesBackupsAdminTest(base.BaseVolumeAdminTest):
    """Test volume backups"""

    @classmethod
    def skip_checks(cls):
        super(VolumesBackupsAdminTest, cls).skip_checks()
        if not CONF.volume_feature_enabled.backup:
            raise cls.skipException("Cinder backup feature disabled")

    def _delete_backup(self, backup_id):
        self.admin_backups_client.delete_backup(backup_id)
        self.admin_backups_client.wait_for_resource_deletion(backup_id)

    def _decode_url(self, backup_url):
        return json.loads(base64.decode_as_text(backup_url))

    def _encode_backup(self, backup):
        retval = json.dumps(backup)
        return base64.encode_as_text(retval)

    def _modify_backup_url(self, backup_url, changes):
        backup = self._decode_url(backup_url)
        backup.update(changes)
        return self._encode_backup(backup)

    @decorators.idempotent_id('a99c54a1-dd80-4724-8a13-13bf58d4068d')
    def test_volume_backup_export_import(self):
        """Test backup export import functionality.

        Cinder allows exporting DB backup information through its API so it can
        be imported back in case of a DB loss.
        """
        volume = self.create_volume()
        # Create backup
        backup_name = data_utils.rand_name(self.__class__.__name__ + '-Backup')
        backup = self.create_backup(volume_id=volume['id'], name=backup_name)
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                volume['id'], 'available')
        self.assertEqual(backup_name, backup['name'])

        # Export Backup
        export_backup = (self.admin_backups_client.export_backup(backup['id'])
                         ['backup-record'])
        self.assertTrue(export_backup['backup_service'].startswith(
                        'cinder.backup.drivers'))

        # NOTE(geguileo): Backups are imported with the same backup id
        # (important for incremental backups among other things), so we cannot
        # import the exported backup information as it is, because that Backup
        # ID already exists.  So we'll fake the data by changing the backup id
        # in the exported backup DB info we have retrieved before importing it
        # back.
        new_id = data_utils.rand_uuid()
        new_url = self._modify_backup_url(
            export_backup['backup_url'], {'id': new_id})

        # Import Backup
        import_backup = self.admin_backups_client.import_backup(
            backup_service=export_backup['backup_service'],
            backup_url=new_url)['backup']

        # NOTE(geguileo): We delete both backups, but only one of those
        # deletions will delete data from the backup back-end because they
        # were both pointing to the same backend data.
        self.addCleanup(self._delete_backup, new_id)
        self.assertEqual(new_id, import_backup['id'])
        waiters.wait_for_volume_resource_status(self.admin_backups_client,
                                                import_backup['id'],
                                                'available')

        # Verify Import Backup
        backups = self.admin_backups_client.list_backups()['backups']
        self.assertIn(new_id, [b['id'] for b in backups])

        # Restore backup
        restore = self.backups_client.restore_backup(backup['id'])['restore']
        self.addCleanup(self.volumes_client.delete_volume,
                        restore['volume_id'])
        self.assertEqual(backup['id'], restore['backup_id'])

        # When restore operation is performed then, backup['id']
        # goes to 'restoring' state so we need to wait for
        # backup['id'] to become 'available'.
        waiters.wait_for_volume_resource_status(
            self.backups_client, backup['id'], 'available')
        waiters.wait_for_volume_resource_status(
            self.volumes_client, restore['volume_id'], 'available')

        # Verify if restored volume is there in volume list
        volumes = self.volumes_client.list_volumes()['volumes']
        self.assertIn(restore['volume_id'], [v['id'] for v in volumes])

    @decorators.idempotent_id('47a35425-a891-4e13-961c-c45deea21e94')
    def test_volume_backup_reset_status(self):
        """Test resetting volume backup status to error"""
        # Create a volume
        volume = self.create_volume()
        # Create a backup
        backup_name = data_utils.rand_name(
            self.__class__.__name__ + '-Backup')
        backup = self.create_backup(volume_id=volume['id'], name=backup_name)
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                volume['id'], 'available')
        self.assertEqual(backup_name, backup['name'])
        # Reset backup status to error
        self.admin_backups_client.reset_backup_status(backup_id=backup['id'],
                                                      status="error")
        waiters.wait_for_volume_resource_status(self.backups_client,
                                                backup['id'], 'error')



# Copyright 2013 OpenStack, Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from tempest.api.network import base
from tempest.common import utils
from tempest.lib import decorators


class ExtensionsTestJSON(base.BaseNetworkTest):
    """Tests the following operations in the Neutron API:

        List all available extensions

    v2.0 of the Neutron API is assumed. It is also assumed that api-extensions
    option is defined in the [network-feature-enabled] section of
    etc/tempest.conf.
    """

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('ef28c7e6-e646-4979-9d67-deb207bc5564')
    def test_list_show_extensions(self):
        """List available extensions and show the detail of each extension"""
        expected_alias = ['security-group', 'l3_agent_scheduler',
                          'ext-gw-mode', 'binding', 'quotas',
                          'agent', 'dhcp_agent_scheduler', 'provider',
                          'router', 'extraroute', 'external-net',
                          'allowed-address-pairs', 'extra_dhcp_opt',
                          'metering', 'dvr']
        expected_alias = [ext for ext in expected_alias if
                          utils.is_extension_enabled(ext, 'network')]
        actual_alias = list()
        extensions = self.network_extensions_client.list_extensions()
        list_extensions = extensions['extensions']
        # Show and verify the details of the available extensions
        for ext in list_extensions:
            ext_name = ext['name']
            ext_alias = ext['alias']
            actual_alias.append(ext['alias'])
            ext_details = self.network_extensions_client.show_extension(
                ext_alias)
            ext_details = ext_details['extension']

            self.assertIsNotNone(ext_details)
            self.assertIn('updated', ext_details.keys())
            self.assertIn('name', ext_details.keys())
            self.assertIn('description', ext_details.keys())
            self.assertIn('links', ext_details.keys())
            self.assertIn('alias', ext_details.keys())
            self.assertEqual(ext_details['name'], ext_name)
            self.assertEqual(ext_details['alias'], ext_alias)
            self.assertEqual(ext_details, ext)
        # Verify if expected extensions are present in the actual list
        # of extensions returned, but only for those that have been
        # enabled via configuration
        for e in expected_alias:
            if utils.is_extension_enabled(e, 'network'):
                self.assertIn(e, actual_alias)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute import base
from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

CONF = config.CONF


class LiveMigrationNegativeTest(base.BaseV2ComputeAdminTest):
    """Negative tests of live migration"""

    @classmethod
    def skip_checks(cls):
        super(LiveMigrationNegativeTest, cls).skip_checks()
        if not CONF.compute_feature_enabled.live_migration:
            raise cls.skipException("Live migration is not enabled")

    def _migrate_server_to(self, server_id, dest_host):
        bmflm = CONF.compute_feature_enabled.block_migration_for_live_migration
        kwargs = dict(host=dest_host, block_migration=bmflm)
        if self.is_requested_microversion_compatible('2.24'):
            kwargs['disk_over_commit'] = False
        self.admin_servers_client.live_migrate_server(server_id, **kwargs)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('7fb7856e-ae92-44c9-861a-af62d7830bcb')
    def test_invalid_host_for_migration(self):
        """Test migrating to an invalid host should not change the status"""
        target_host = data_utils.rand_name('host')
        server = self.create_test_server(wait_until="ACTIVE")

        self.assertRaises(lib_exc.BadRequest, self._migrate_server_to,
                          server['id'], target_host)
        waiters.wait_for_server_status(self.servers_client, server['id'],
                                       'ACTIVE')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('6e2f94f5-2ee8-4830-bef5-5bc95bb0795b')
    def test_live_block_migration_suspended(self):
        """Test migrating a suspended server should not change the status"""
        server = self.create_test_server(wait_until="ACTIVE")

        self.admin_servers_client.suspend_server(server['id'])
        waiters.wait_for_server_status(self.servers_client,
                                       server['id'], 'SUSPENDED')

        destination_host = self.get_host_other_than(server['id'])

        self.assertRaises(lib_exc.Conflict, self._migrate_server_to,
                          server['id'], destination_host)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute import base
from tempest.common import waiters
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc


class ListServersNegativeTestJSON(base.BaseV2ComputeTest):
    """Negative tests of listing servers"""

    create_default_network = True

    @classmethod
    def setup_clients(cls):
        super(ListServersNegativeTestJSON, cls).setup_clients()
        cls.client = cls.servers_client

    @classmethod
    def resource_setup(cls):
        super(ListServersNegativeTestJSON, cls).resource_setup()

        # The following servers are created for use
        # by the test methods in this class. These
        # servers are cleaned up automatically in the
        # tearDownClass method of the super-class.
        body = cls.create_test_server(wait_until='ACTIVE', min_count=3)

        # delete one of the created servers
        cls.deleted_id = body['server']['id']
        cls.client.delete_server(cls.deleted_id)
        waiters.wait_for_server_termination(cls.client, cls.deleted_id)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('24a26f1a-1ddc-4eea-b0d7-a90cc874ad8f')
    def test_list_servers_with_a_deleted_server(self):
        """Test that deleted servers do not show by default in list servers"""
        # List servers and verify server not returned
        body = self.client.list_servers()
        servers = body['servers']
        actual = [srv for srv in servers
                  if srv['id'] == self.deleted_id]
        self.assertEmpty(actual)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('ff01387d-c7ad-47b4-ae9e-64fa214638fe')
    def test_list_servers_by_non_existing_image(self):
        """Test listing servers for a non existing image returns empty list"""
        body = self.client.list_servers(image='non_existing_image')
        servers = body['servers']
        self.assertEmpty(servers)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('5913660b-223b-44d4-a651-a0fbfd44ca75')
    def test_list_servers_by_non_existing_flavor(self):
        """Test listing servers by non existing flavor returns empty list"""
        body = self.client.list_servers(flavor='non_existing_flavor')
        servers = body['servers']
        self.assertEmpty(servers)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('e2c77c4a-000a-4af3-a0bd-629a328bde7c')
    def test_list_servers_by_non_existing_server_name(self):
        """Test listing servers for a non existent server name

        Listing servers for a non existent server name should return empty
        list.
        """

        body = self.client.list_servers(name='non_existing_server_name')
        servers = body['servers']
        self.assertEmpty(servers)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('fcdf192d-0f74-4d89-911f-1ec002b822c4')
    def test_list_servers_status_non_existing(self):
        """Test listing servers with non existing status

        When invalid status is specified, up to microversion 2.37,
        an empty list is returned, and starting from microversion 2.38,
        a 400 error is returned in that case.
        """

        if self.is_requested_microversion_compatible('2.37'):
            body = self.client.list_servers(status='non_existing_status')
            servers = body['servers']
            self.assertEmpty(servers)
        else:
            self.assertRaises(lib_exc.BadRequest, self.client.list_servers,
                              status='non_existing_status')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('d47c17fb-eebd-4287-8e95-f20a7e627b18')
    def test_list_servers_by_limits_greater_than_actual_count(self):
        """Test listing servers by limit greater than actual count

        Listing servers by limit greater than actual count should return
        all servers.
        """

        # Gather the complete list of servers in the project for reference
        full_list = self.client.list_servers()['servers']
        # List servers by specifying a greater value for limit
        limit = len(full_list) + 100
        body = self.client.list_servers(limit=limit)
        self.assertEqual(len(full_list), len(body['servers']))

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('679bc053-5e70-4514-9800-3dfab1a380a6')
    def test_list_servers_by_limits_pass_string(self):
        """Test listing servers by non-integer limit should fail"""
        self.assertRaises(lib_exc.BadRequest, self.client.list_servers,
                          limit='testing')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('62610dd9-4713-4ee0-8beb-fd2c1aa7f950')
    def test_list_servers_by_limits_pass_negative_value(self):
        """Test listing servers by negative limit should fail"""
        self.assertRaises(lib_exc.BadRequest, self.client.list_servers,
                          limit=-1)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('87d12517-e20a-4c9c-97b6-dd1628d6d6c9')
    def test_list_servers_by_changes_since_invalid_date(self):
        """Test listing servers by invalid changes-since format should fail"""
        params = {'changes-since': '2011/01/01'}
        self.assertRaises(lib_exc.BadRequest, self.client.list_servers,
                          **params)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('74745ad8-b346-45b5-b9b8-509d7447fc1f')
    def test_list_servers_by_changes_since_future_date(self):
        """Test listing servers by a future changes-since date

        Return an empty list when a date in the future is passed as
        changes-since value.
        """

        # updated_at field may haven't been set at the point in the boot
        # process where build_request still exists, so add
        # {'status': 'ACTIVE'} along with changes-since as filter.
        changes_since = {'changes-since': '2051-01-01T12:34:00Z',
                         'status': 'ACTIVE'}
        body = self.client.list_servers(**changes_since)
        self.assertEmpty(body['servers'])

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('93055106-2d34-46fe-af68-d9ddbf7ee570')
    def test_list_servers_detail_server_is_deleted(self):
        """Test listing servers detail should not contain deleted server"""
        body = self.client.list_servers(detail=True)
        servers = body['servers']
        actual = [srv for srv in servers
                  if srv['id'] == self.deleted_id]
        self.assertEmpty(actual)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from tempest.lib.api_schema.response.compute.v2_1 import servers as servers2_1
from tempest.lib.api_schema.response.compute.v2_63 import servers as servers263


###########################################################################
#
# 2.70:
#
# Exposes virtual device tags for volume attachments and virtual interfaces
# (ports). A tag parameter is added to the response body for the following
# APIs:
#
# Volumes
#
# - GET /servers/{server_id}/os-volume_attachments (list)
# - GET /servers/{server_id}/os-volume_attachments/{volume_id} (show)
# - POST /servers/{server_id}/os-volume_attachments (attach)
#
# Ports
#
# - GET /servers/{server_id}/os-interface (list)
# - GET /servers/{server_id}/os-interface/{port_id} (show)
# - POST /servers/{server_id}/os-interface (attach)
#
###########################################################################

attach_volume = copy.deepcopy(servers2_1.attach_volume)
attach_volume['response_body']['properties']['volumeAttachment'][
    'properties'].update({'tag': {'type': ['string', 'null']}})
attach_volume['response_body']['properties']['volumeAttachment'][
    'required'].append('tag')

show_volume_attachment = copy.deepcopy(servers2_1.show_volume_attachment)
show_volume_attachment['response_body']['properties']['volumeAttachment'][
    'properties'].update({'tag': {'type': ['string', 'null']}})
show_volume_attachment['response_body']['properties'][
    'volumeAttachment']['required'].append('tag')

list_volume_attachments = copy.deepcopy(servers2_1.list_volume_attachments)
list_volume_attachments['response_body']['properties']['volumeAttachments'][
    'items']['properties'].update({'tag': {'type': ['string', 'null']}})
list_volume_attachments['response_body']['properties'][
    'volumeAttachments']['items']['required'].append('tag')

# TODO(mriedem): Handle the os-interface changes when there is a test that
# needs them from this microversion onward.

# NOTE(lajoskatona): Below are the unchanged schema in this microversion. We
# need to keep this schema in this file to have the generic way to select the
# right schema based on self.schema_versions_info mapping in service client.
# ****** Schemas unchanged since microversion 2.63 ***
list_servers_detail = copy.deepcopy(servers263.list_servers_detail)
rebuild_server = copy.deepcopy(servers263.rebuild_server)
rebuild_server_with_admin_pass = copy.deepcopy(
    servers263.rebuild_server_with_admin_pass)
update_server = copy.deepcopy(servers263.update_server)
get_server = copy.deepcopy(servers263.get_server)
list_servers = copy.deepcopy(servers263.list_servers)
show_server_diagnostics = copy.deepcopy(servers263.show_server_diagnostics)
get_remote_consoles = copy.deepcopy(servers263.get_remote_consoles)
list_tags = copy.deepcopy(servers263.list_tags)
update_all_tags = copy.deepcopy(servers263.update_all_tags)
delete_all_tags = copy.deepcopy(servers263.delete_all_tags)
check_tag_existence = copy.deepcopy(servers263.check_tag_existence)
update_tag = copy.deepcopy(servers263.update_tag)
delete_tag = copy.deepcopy(servers263.delete_tag)



# Copyright 2013 Huawei Technologies Co.,LTD.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute import base
from tempest.common import tempest_fixtures as fixtures
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc


class AggregatesAdminNegativeTestJSON(base.BaseV2ComputeAdminTest):
    """Tests Aggregates API that require admin privileges"""

    @classmethod
    def setup_clients(cls):
        super(AggregatesAdminNegativeTestJSON, cls).setup_clients()
        cls.client = cls.os_admin.aggregates_client
        cls.services_client = cls.os_admin.services_client

    @classmethod
    def resource_setup(cls):
        super(AggregatesAdminNegativeTestJSON, cls).resource_setup()
        cls.aggregate_name_prefix = 'test_aggregate'

        svc_list = cls.services_client.list_services(
            binary='nova-compute')['services']
        cls.hosts = [v['host'] for v in svc_list
                     if v['status'] == 'enabled' and v['state'] == 'up']

    def _create_test_aggregate(self):
        aggregate_name = data_utils.rand_name(self.aggregate_name_prefix)
        aggregate = (self.client.create_aggregate(name=aggregate_name)
                     ['aggregate'])
        self.addCleanup(self.client.delete_aggregate, aggregate['id'])
        return aggregate

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('86a1cb14-da37-4a70-b056-903fd56dfe29')
    def test_aggregate_create_as_user(self):
        """Regular user is not allowed to create an aggregate"""
        aggregate_name = data_utils.rand_name(self.aggregate_name_prefix)
        self.assertRaises(lib_exc.Forbidden,
                          self.aggregates_client.create_aggregate,
                          name=aggregate_name)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('3b8a1929-3793-4e92-bcb4-dfa572ee6c1d')
    def test_aggregate_create_aggregate_name_length_less_than_1(self):
        """The length of aggregate name should >=1"""
        self.assertRaises(lib_exc.BadRequest,
                          self.client.create_aggregate,
                          name='')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('4c194563-543b-4e70-a719-557bbe947fac')
    def test_aggregate_create_aggregate_name_length_exceeds_255(self):
        """The length of aggregate name should <=255"""
        aggregate_name = 'a' * 256
        self.assertRaises(lib_exc.BadRequest,
                          self.client.create_aggregate,
                          name=aggregate_name)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('9c23a291-b0b1-487b-b464-132e061151b3')
    def test_aggregate_create_with_existent_aggregate_name(self):
        """Creating an aggregate with existent aggregate name is forbidden"""
        aggregate = self._create_test_aggregate()
        self.assertRaises(lib_exc.Conflict,
                          self.client.create_aggregate,
                          name=aggregate['name'])

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('cd6de795-c15d-45f1-8d9e-813c6bb72a3d')
    def test_aggregate_delete_as_user(self):
        """Regular user is not allowed to delete an aggregate"""
        aggregate = self._create_test_aggregate()
        self.assertRaises(lib_exc.Forbidden,
                          self.aggregates_client.delete_aggregate,
                          aggregate['id'])

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('b7d475a6-5dcd-4ff4-b70a-cd9de66a6672')
    def test_aggregate_list_as_user(self):
        """Regular user is not allowed to list aggregates"""
        self.assertRaises(lib_exc.Forbidden,
                          self.aggregates_client.list_aggregates)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('557cad12-34c9-4ff4-95f0-22f0dfbaf7dc')
    def test_aggregate_get_details_as_user(self):
        """Regular user is not allowed to get aggregate details"""
        aggregate = self._create_test_aggregate()
        self.assertRaises(lib_exc.Forbidden,
                          self.aggregates_client.show_aggregate,
                          aggregate['id'])

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('c74f4bf1-4708-4ff2-95a0-f49eaca951bd')
    def test_aggregate_delete_with_invalid_id(self):
        """Delete an aggregate with invalid id should raise exceptions"""
        self.assertRaises(lib_exc.NotFound,
                          self.client.delete_aggregate, -1)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('3c916244-2c46-49a4-9b55-b20bb0ae512c')
    def test_aggregate_get_details_with_invalid_id(self):
        """Get aggregate details with invalid id should raise exceptions"""
        self.assertRaises(lib_exc.NotFound,
                          self.client.show_aggregate, -1)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('0ef07828-12b4-45ba-87cc-41425faf5711')
    def test_aggregate_add_non_exist_host(self):
        """Adding a non-exist host to an aggregate should fail"""
        while True:
            non_exist_host = data_utils.rand_name('nonexist_host')
            if non_exist_host not in self.hosts:
                break
        aggregate = self._create_test_aggregate()
        self.assertRaises(lib_exc.NotFound, self.client.add_host,
                          aggregate['id'], host=non_exist_host)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('7324c334-bd13-4c93-8521-5877322c3d51')
    def test_aggregate_add_host_as_user(self):
        """Regular user is not allowed to add a host to an aggregate"""
        aggregate = self._create_test_aggregate()
        self.assertRaises(lib_exc.Forbidden,
                          self.aggregates_client.add_host,
                          aggregate['id'], host=self.hosts[0])

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('19dd44e1-c435-4ee1-a402-88c4f90b5950')
    def test_aggregate_add_existent_host(self):
        """Adding already existing host to aggregate should fail"""
        self.useFixture(fixtures.LockFixture('availability_zone'))
        aggregate = self._create_test_aggregate()

        self.client.add_host(aggregate['id'], host=self.hosts[0])
        self.addCleanup(self.client.remove_host, aggregate['id'],
                        host=self.hosts[0])

        self.assertRaises(lib_exc.Conflict, self.client.add_host,
                          aggregate['id'], host=self.hosts[0])

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('7a53af20-137a-4e44-a4ae-e19260e626d9')
    def test_aggregate_remove_host_as_user(self):
        """Regular user is not allowed to remove a host from an aggregate"""
        self.useFixture(fixtures.LockFixture('availability_zone'))
        aggregate = self._create_test_aggregate()

        self.client.add_host(aggregate['id'], host=self.hosts[0])
        self.addCleanup(self.client.remove_host, aggregate['id'],
                        host=self.hosts[0])

        self.assertRaises(lib_exc.Forbidden,
                          self.aggregates_client.remove_host,
                          aggregate['id'], host=self.hosts[0])

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('95d6a6fa-8da9-4426-84d0-eec0329f2e4d')
    def test_aggregate_remove_nonexistent_host(self):
        """Removing not existing host from aggregate should fail"""
        aggregate = self._create_test_aggregate()

        self.assertRaises(lib_exc.NotFound, self.client.remove_host,
                          aggregate['id'], host='nonexist_host')



# Copyright (c) 2016 Hewlett-Packard Enterprise Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

from tempest.lib.services.volume.v2.availability_zone_client \
    import AvailabilityZoneClient
from tempest.lib.services.volume.v2.backups_client import BackupsClient
from tempest.lib.services.volume.v2.capabilities_client import \
    CapabilitiesClient
from tempest.lib.services.volume.v2.encryption_types_client import \
    EncryptionTypesClient
from tempest.lib.services.volume.v2.extensions_client import ExtensionsClient
from tempest.lib.services.volume.v2.hosts_client import HostsClient
from tempest.lib.services.volume.v2.limits_client import LimitsClient
from tempest.lib.services.volume.v2.qos_client import QosSpecsClient
from tempest.lib.services.volume.v2.quota_classes_client import \
    QuotaClassesClient
from tempest.lib.services.volume.v2.quotas_client import QuotasClient
from tempest.lib.services.volume.v2.scheduler_stats_client import \
    SchedulerStatsClient
from tempest.lib.services.volume.v2.services_client import ServicesClient
from tempest.lib.services.volume.v2.snapshot_manage_client import \
    SnapshotManageClient
from tempest.lib.services.volume.v2.snapshots_client import SnapshotsClient
from tempest.lib.services.volume.v2.transfers_client import TransfersClient
from tempest.lib.services.volume.v2.types_client import TypesClient
from tempest.lib.services.volume.v2.volume_manage_client import \
    VolumeManageClient
from tempest.lib.services.volume.v2.volumes_client import VolumesClient

__all__ = ['AvailabilityZoneClient', 'BackupsClient', 'EncryptionTypesClient',
           'ExtensionsClient', 'HostsClient', 'QosSpecsClient', 'QuotasClient',
           'ServicesClient', 'SnapshotsClient', 'TypesClient', 'VolumesClient',
           'LimitsClient', 'CapabilitiesClient', 'SchedulerStatsClient',
           'SnapshotManageClient', 'VolumeManageClient', 'TransfersClient',
           'QuotaClassesClient']
